@@ -31,20 +31,58 @@ import gobject
 gobject.threads_init()
 import gst
 import gst.pbutils
+from gst.pbutils import INSTALL_PLUGINS_SUCCESS, \
+        INSTALL_PLUGINS_PARTIAL_SUCCESS, INSTALL_PLUGINS_USER_ABORT, \
+        INSTALL_PLUGINS_STARTED_OK
 import tempfile
-from base64 import urlsafe_b64encode
+import hashlib
 
 from pitivi.log.loggable import Loggable
 from pitivi.factories.file import FileSourceFactory, PictureFileSourceFactory
 from pitivi.stream import get_stream_for_pad
 from pitivi.signalinterface import Signallable
-from pitivi.stream import VideoStream
+from pitivi.stream import VideoStream, TextStream
+from pitivi.settings import xdg_cache_home
 
 # FIXME: We need to store more information regarding streams
 # i.e. remember the path took to get to a raw stream, and figure out
 # what encoded format it is
 # We will need that in order to create proper Stream objects.
 
+class EOSSir(gst.Element):
+    __gstdetails__ = (
+        "EOSSir",
+        "Generic",
+        "pushes EOS after the first buffer",
+        "Alessandro Decina <alessandro.d@gmail.com>"
+        )
+
+    srctemplate = gst.PadTemplate("src", gst.PAD_SRC,
+            gst.PAD_ALWAYS, gst.Caps("ANY"))
+    sinktemplate = gst.PadTemplate("sink", gst.PAD_SINK,
+            gst.PAD_ALWAYS, gst.Caps("ANY"))
+
+    __gsttemplates__ = (srctemplate, sinktemplate)
+
+    def __init__(self):
+        gst.Element.__init__(self)
+
+        self.sinkpad = gst.Pad(self.sinktemplate, "sink")
+        self.sinkpad.set_chain_function(self.chain)
+        self.add_pad(self.sinkpad)
+
+        self.srcpad = gst.Pad(self.srctemplate, "src")
+        self.add_pad(self.srcpad)
+
+    def chain(self, pad, buf):
+        ret = self.srcpad.push(buf)
+        if ret == gst.FLOW_OK:
+            self.info("pushed, doing EOS")
+            self.srcpad.push_event(gst.event_new_eos())
+
+        return ret
+gobject.type_register(EOSSir)
+
 class Discoverer(Signallable, Loggable):
     """
     Queues requests to discover information about given files.
@@ -87,9 +125,12 @@ class Discoverer(Signallable, Loggable):
         self.error = None
         self.error_detail = None
         self.unfixed_pads = 0
+        self.unknown_pads = 0
         self.missing_plugin_messages = []
         self.dynamic_elements = []
         self.thumbnails = {}
+        self.missing_plugin_details = []
+        self.missing_plugin_descriptions = []
 
     def _resetPipeline(self):
         # finish current, cleanup
@@ -98,9 +139,9 @@ class Discoverer(Signallable, Loggable):
             self.bus = None
 
         if self.pipeline is not None:
-            self.info("before setting to NULL")
+            self.debug("before setting to NULL")
             res = self.pipeline.set_state(gst.STATE_NULL)
-            self.info("after setting to NULL : %s", res)
+            self.debug("after setting to NULL : %s", res)
 
         for element in self.dynamic_elements:
             self.pipeline.remove(element)
@@ -152,35 +193,110 @@ class Discoverer(Signallable, Loggable):
         if not self.missing_plugin_messages:
             return False
 
-        missing_plugin_details = []
-        missing_plugin_descriptions = []
         for message in self.missing_plugin_messages:
             detail = \
                     gst.pbutils.missing_plugin_message_get_installer_detail(message)
             description = \
                     gst.pbutils.missing_plugin_message_get_description(message)
 
-            missing_plugin_details.append(detail)
-            missing_plugin_descriptions.append(description)
+            self.missing_plugin_details.append(detail)
+            self.missing_plugin_descriptions.append(description)
 
-        result = self.emit('missing-plugins', self.current_uri,
-                missing_plugin_details, missing_plugin_descriptions)
+        return True
 
-        if result == gst.pbutils.INSTALL_PLUGINS_STARTED_OK:
-            # don't emit an error yet
-            self.error = None
-            self.error_detail = None
-            res = True
+    def _installMissingPluginsCallback(self, result, factory):
+        rescan = False
+
+        if result in (INSTALL_PLUGINS_SUCCESS,
+                INSTALL_PLUGINS_PARTIAL_SUCCESS):
+            gst.update_registry()
+            rescan = True
+        elif result == INSTALL_PLUGINS_USER_ABORT \
+                and factory.getOutputStreams():
+            self._emitDone(factory)
         else:
-            if self.error is None:
-                self.error = _('Missing plugins:\n%s') % \
-                             '\n'.join(missing_plugin_descriptions)
-                self.error_detail = ''
-            res = False
+            self._emitErrorMissingPlugins()
 
-        return res
+        self._finishAnalysisAfterResult(rescan=rescan)
 
-    def _finishAnalysis(self):
+    def _emitError(self):
+        self.debug("emitting error %s, %s, %s",
+                self.current_uri, self.error, self.error_detail)
+        self.emit("discovery-error", self.current_uri, self.error, self.error_detail)
+
+    def _emitErrorMissingPlugins(self):
+        self.error = _("Missing plugins:\n%s") % \
+                "\n".join(self.missing_plugin_descriptions)
+        self.error_detail = ""
+        self._emitError()
+
+    def _emitDone(self, factory):
+        self.emit("discovery-done", self.current_uri, factory)
+
+    def _emitResult(self):
+        missing_plugins = bool(self.missing_plugin_details)
+        # we got a gst error, error out ASAP
+        if not missing_plugins and self.error:
+            self._emitError()
+            return True
+
+        have_video, have_audio, have_image = self._getCurrentStreamTypes()
+        missing_plugins = bool(self.missing_plugin_details)
+
+        if not self.current_streams and not missing_plugins:
+            # woot, nothing decodable
+            self.error = _('Can not decode file.')
+            self.error_detail = _("The given file does not contain audio, "
+                    "video or picture streams.")
+            self._emitError()
+            return True
+
+        # construct the factory with the streams we found
+        if have_image and self.current_duration == gst.CLOCK_TIME_NONE:
+            factory = PictureFileSourceFactory(self.current_uri)
+        else:
+            factory = FileSourceFactory(self.current_uri)
+
+        factory.duration = self.current_duration
+        for stream in self.current_streams:
+            factory.addOutputStream(stream)
+
+        if not missing_plugins:
+            # make sure that we could query the duration (if it's an image, we
+            # assume it's got infinite duration)
+            is_image = have_image and len(self.current_streams) == 1
+            if self.current_duration == gst.CLOCK_TIME_NONE and not is_image:
+                self.error =_("Could not establish the duration of the file.")
+                self.error_detail = _("This clip seems to be in a format "
+                        "which cannot be accessed in a random fashion.")
+                self._emitError()
+                return True
+
+            self._emitDone(factory)
+            return True
+
+        def callback(result):
+            self._installMissingPluginsCallback(result, factory)
+
+        res = self.emit("missing-plugins", self.current_uri, factory,
+                self.missing_plugin_details,
+                self.missing_plugin_descriptions,
+                callback)
+        if res is None or res != INSTALL_PLUGINS_STARTED_OK:
+            # no missing-plugins handlers
+            if factory.getOutputStreams():
+                self._emitDone(factory)
+            else:
+                self._emitErrorMissingPlugins()
+
+            return True
+
+
+        # plugins are being installed, processing will continue when
+        # self._installMissingPluginsCallback is called by the application
+        return False
+
+    def _finishAnalysis(self, reason):
         """
         Call this method when the current file is analyzed
         This method will wrap-up the analyzis and call the next analysis if needed
@@ -188,49 +304,23 @@ class Discoverer(Signallable, Loggable):
         if self.timeout_id:
             self._removeTimeout()
 
-        missing_plugins = self._checkMissingPlugins()
+        self.info("analysys finished, reason %s", reason)
+
+        # check if there are missing plugins before calling _resetPipeline as we
+        # are going to pop messagess off the bus
+        self._checkMissingPlugins()
         self._resetPipeline()
-        if not self.current_streams and self.error is None:
-            # EOS and no decodable streams?
-            self.error = _('No streams found')
-            self.error_detail = ""
 
-        if len(self.current_streams) == 1:
-            stream = self.current_streams[0]
-            is_image = isinstance(stream, VideoStream) and stream.is_image
-        else:
-            is_image = False
-
-        if self.error:
-            self.emit('discovery-error', self.current_uri, self.error, self.error_detail)
-        elif self.current_duration == gst.CLOCK_TIME_NONE and not is_image:
-            self.emit('discovery-error', self.current_uri,
-                      _("Could not establish the duration of the file."),
-                      _("This clip seems to be in a format which cannot be accessed in a random fashion."))
-        else:
-            have_video, have_audio, have_image = self._getCurrentStreamTypes()
-            if have_video or have_audio:
-                factory = FileSourceFactory(self.current_uri)
-            elif have_image:
-                factory = PictureFileSourceFactory(self.current_uri)
-            else:
-                # woot, nothing decodable
-                self.error = _('Can not decode file.')
-                self.error_detail = _('The given file does not contain audio, video or picture streams.')
-                factory = None
-
-            if factory is not None:
-                factory.duration = self.current_duration
-
-                for stream in self.current_streams:
-                    factory.addOutputStream(stream)
-
-            self.emit('discovery-done', self.current_uri, factory)
+        # emit discovery-done, discovery-error or missing-plugins
+        if self._emitResult():
+            self._finishAnalysisAfterResult()
 
+    def _finishAnalysisAfterResult(self, rescan=False):
         self.info("Cleaning up after finished analyzing %s", self.current_uri)
         self._resetState()
 
-        self.queue.pop(0)
+        if not rescan:
+            self.queue.pop(0)
         # restart an analysis if there's more...
         if self.queue:
             self._scheduleAnalysis()
@@ -245,7 +335,7 @@ class Discoverer(Signallable, Loggable):
         if not self.error:
             self.error = _('Timeout while analyzing file.')
             self.error_detail = _('Analyzing the file took too long.')
-        self._finishAnalysis()
+        self._finishAnalysis("timeout")
 
         return False
 
@@ -295,6 +385,7 @@ class Discoverer(Signallable, Loggable):
             dbin = gst.element_factory_make("decodebin", "dbin")
 
         dbin.connect("new-decoded-pad", self._newDecodedPadCb)
+        dbin.connect("unknown-type", self._unknownType)
 
         return dbin
 
@@ -320,7 +411,7 @@ class Discoverer(Signallable, Loggable):
         # create the source element
         source = self._createSource()
         if source is None:
-            self._finishAnalysis()
+            self._finishAnalysis("no source")
             return False
 
         # create decodebin(2)
@@ -340,7 +431,7 @@ class Discoverer(Signallable, Loggable):
             if not self.error:
                 self.error = _("Pipeline didn't want to go to PAUSED.")
             self.info("Pipeline didn't want to go to PAUSED")
-            self._finishAnalysis()
+            self._finishAnalysis("failure going to PAUSED")
 
             return False
 
@@ -350,9 +441,9 @@ class Discoverer(Signallable, Loggable):
         return False
 
     def _busMessageEosCb(self, unused_bus, message):
-        self.log("got EOS")
+        self.debug("got EOS")
 
-        self._finishAnalysis()
+        self._finishAnalysis("EOS")
 
     def _busMessageErrorCb(self, unused_bus, message):
         gerror, detail = message.parse_error()
@@ -361,10 +452,10 @@ class Discoverer(Signallable, Loggable):
             # don't clobber existing errors
             return
 
-        self.error = _("An internal error occured while analyzing this file: %s") % gerror.message
+        self.error = _("An internal error occurred while analyzing this file: %s") % gerror.message
         self.error_detail = detail
 
-        self._finishAnalysis()
+        self._finishAnalysis("ERROR")
 
     def _busMessageElementCb(self, unused_bus, message):
         self.debug("Element message %s", message.structure.to_string())
@@ -374,7 +465,7 @@ class Discoverer(Signallable, Loggable):
                 self.error = _("File contains a redirection to another clip.")
                 self.error_detail = _("PiTiVi currently does not handle redirection files.")
 
-            self._finishAnalysis()
+            self._finishAnalysis("redirect")
             return
 
         if gst.pbutils.is_missing_plugin_message(message):
@@ -394,13 +485,17 @@ class Discoverer(Signallable, Loggable):
         if prev == gst.STATE_READY and new == gst.STATE_PAUSED and \
                 pending == gst.STATE_VOID_PENDING:
             have_video, have_audio, have_image = self._getCurrentStreamTypes()
-            if self.unfixed_pads or have_video or have_image:
+            if self.unfixed_pads or self.unknown_pads or have_video or have_image:
                 # go to PLAYING to generate the thumbnails
-                self.pipeline.set_state(gst.STATE_PLAYING)
+                if self.pipeline.set_state(gst.STATE_PLAYING) == gst.STATE_CHANGE_FAILURE:
+                    if not self.error:
+                        self.error = _("Pipeline didn't want to go to PLAYING.")
+                    self.info("Pipeline didn't want to go to PAUSED")
+                    self._finishAnalysis("failure going to PAUSED")
             elif self.unfixed_pads == 0:
                 # check for unfixed_pads until elements are fixed to do
                 # negotiation before pushing in band data
-                self._finishAnalysis()
+                self._finishAnalysis("got to PAUSED and no unfixed pads")
 
     def _busMessageTagCb(self, unused_bus, message):
         self.debug("Got tags %s", message.structure.to_string())
@@ -414,18 +509,110 @@ class Discoverer(Signallable, Loggable):
                 if format == gst.FORMAT_TIME:
                     self.current_duration = duration
 
-    def _getThumbnailFilenameFromPad(self, pad):
+    def _gettempdir(self):
         tmp = tempfile.gettempdir()
-        name = '%s.%s' % (self.current_uri, pad.get_name())
-        name = urlsafe_b64encode(name) + '.png'
-        filename = os.path.join(tmp, name)
+        tmp = os.path.join(tmp, 'pitivi-%s' % os.getenv('USER'))
+        if not os.path.exists(tmp):
+            os.mkdir(tmp)
+        return tmp
+
+    def _getThumbnailFilenameFromPad(self, pad):
+        base = xdg_cache_home()
+        name = self.current_uri
+        md5sum = hashlib.md5()
+        md5sum.update(self.current_uri)
+        name = md5sum.hexdigest() + '.png'
+        directory = os.path.join(base, "pitivi")
+        try:
+            os.makedirs(directory)
+        except OSError, e:
+            # 17 = file exists
+            if e.errno != 17:
+                raise
+        filename = os.path.join(base, "pitivi", name)
 
         return filename
 
+    def _videoPadSeekCb(self, pad):
+        try:
+            duration = self.pipeline.query_duration(gst.FORMAT_TIME)[0]
+        except gst.QueryError:
+            duration = 0
+
+        self.debug("doing thumbnail seek at %s", gst.TIME_ARGS(duration))
+
+        if duration:
+            self.pipeline.seek_simple(gst.FORMAT_TIME,
+                    gst.SEEK_FLAG_FLUSH, duration / 3)
+
+        pad.set_blocked_async(False, self._videoPadBlockCb)
+
+    def _videoPadBlockCb(self, pad, blocked):
+        self.debug("video pad blocked: %s" % blocked)
+        if blocked:
+            gobject.timeout_add(0, self._videoPadSeekCb, pad)
+
+    def _addVideoBufferProbe(self, pad):
+        closure = {}
+        closure['probe_id'] = pad.add_buffer_probe(self._videoBufferProbeCb,
+                closure)
+
+    def _removeVideoBufferProbe(self, pad, closure):
+        pad.remove_buffer_probe(closure['probe_id'])
+
+    def _videoBufferProbeCb(self, pad, buf, closure):
+        self.log("video buffer probe for pad %s", pad)
+        self._removeVideoBufferProbe(pad, closure)
+
+        pad.set_blocked_async(True, self._videoPadBlockCb)
+
+        return False
+
+    def _padEventProbeCb(self, pad, event):
+        self.log("got event %s from src %s on pad %s",
+                event.type, event.src, pad)
+
+        return True
+
+    def _padBufferProbeCb(self, pad, buf):
+        self.debug("got buffer on pad %s", pad)
+
+        return True
+
+    def _addPadProbes(self, pad):
+        pad.add_event_probe(self._padEventProbeCb)
+        pad.add_buffer_probe(self._padBufferProbeCb)
+
     def _newVideoPadCb(self, pad):
         """ a new video pad was found """
         self.debug("pad %r", pad)
 
+        self._addPadProbes(pad)
+
+        thumbnail = self._getThumbnailFilenameFromPad(pad)
+        self.thumbnails[pad] = thumbnail
+        have_thumbnail = os.path.exists(thumbnail)
+
+        if have_thumbnail:
+            self.debug("we already have a thumbnail %s for %s", thumbnail, pad)
+            sink = gst.element_factory_make("fakesink")
+            # use this and not fakesink.props.num_buffers = 1 to avoid some
+            # not-expected errors when discovering pictures
+            eossir = EOSSir()
+            self.dynamic_elements.extend([eossir, sink])
+            self.pipeline.add(eossir, sink)
+            eossir.set_state(gst.STATE_PLAYING)
+            sink.set_state(gst.STATE_PLAYING)
+
+            pad.link(eossir.get_pad("sink"))
+            eossir.link(sink)
+
+            return
+
+        stream = get_stream_for_pad(pad)
+        if isinstance(stream, VideoStream) and not stream.is_image:
+            self._addVideoBufferProbe(pad)
+
         queue = gst.element_factory_make("queue")
         queue.props.max_size_bytes = 5 * 1024 * 1024
         queue.props.max_size_time = 5 * gst.SECOND
@@ -433,24 +620,32 @@ class Discoverer(Signallable, Loggable):
         vscale.props.method = 0
         csp = gst.element_factory_make("ffmpegcolorspace")
         pngenc = gst.element_factory_make("pngenc")
+        pngenc.props.snapshot = True
         pngsink = gst.element_factory_make("filesink")
-        self.thumbnails[pad] = thumbnail = self._getThumbnailFilenameFromPad(pad)
         pngsink.props.location = thumbnail
 
         self.dynamic_elements.extend([queue, vscale, csp, pngenc, pngsink])
 
         self.pipeline.add(queue, vscale, csp, pngenc, pngsink)
-        queue.link(vscale)
-        vscale.link(csp, gst.Caps("video/x-raw-rgb,width=[1,96],height=[1,96];video/x-raw-yuv,width=[1,96],height=[1,96]"))
-        gst.element_link_many(csp, pngenc, pngsink)
+        gst.element_link_many(queue, csp, vscale)
+        vscale.link(pngenc, gst.Caps("video/x-raw-rgb,width=[1,96],height=[1,96];video/x-raw-yuv,width=[1,96],height=[1,96]"))
+        gst.element_link_many(pngenc, pngsink)
         pad.link(queue.get_pad("sink"))
 
         for element in [queue, vscale, csp, pngenc, pngsink]:
             element.sync_state_with_parent()
 
     def _newPadCb(self, pad):
+        stream = get_stream_for_pad(pad)
+        if isinstance(stream, TextStream):
+            self.info("skipping subtitle pad")
+            return
+
+        self._addPadProbes(pad)
+
         queue = gst.element_factory_make('queue')
         fakesink = gst.element_factory_make('fakesink')
+        fakesink.props.num_buffers = 1
         self.dynamic_elements.append(queue)
         self.dynamic_elements.append(fakesink)
 
@@ -466,12 +661,16 @@ class Discoverer(Signallable, Loggable):
             ghost = pad
 
         caps = pad.props.caps
+        self.debug("pad caps notify %s", caps)
         if caps is None or not caps.is_fixed():
             return
 
         pad.disconnect_by_func(self._capsNotifyCb)
 
+        self.info("got fixed caps for pad %s", pad)
+
         self.unfixed_pads -= 1
+        self.debug("unfixed pads %d", self.unfixed_pads)
         stream = self._addStreamFromPad(ghost)
         if isinstance(stream, VideoStream):
             stream.thumbnail = self.thumbnails[ghost]
@@ -486,13 +685,13 @@ class Discoverer(Signallable, Loggable):
             self._newPadCb(pad)
 
         # try to get the duration
-        # NOTE: this gets the duration only once, usually for the first stream.
-        # Demuxers don't seem to implement per stream duration queries anyway.
         self._maybeQueryDuration(pad)
 
         caps = pad.props.caps
 
         if caps is not None and caps.is_fixed():
+            self.debug("got fixed caps for pad %s", pad)
+
             stream = self._addStreamFromPad(pad)
             if isinstance(stream, VideoStream):
                 stream.thumbnail = self.thumbnails[pad]
@@ -508,8 +707,16 @@ class Discoverer(Signallable, Loggable):
             else:
                 pad.connect("notify::caps", self._capsNotifyCb)
             self.unfixed_pads += 1
+            self.debug("unfixed pads %d", self.unfixed_pads)
+
+    def _unknownType(self, decodebin, pad, caps):
+        # decodebin2 sends ASYNC_DONE when it finds an unknown type so we have
+        # to deal with that...
+        self.unknown_pads += 1
 
     def _addStreamFromPad(self, pad):
+        self._maybeQueryDuration(pad)
+        self.debug("adding stream from pad %s caps %s", pad, pad.props.caps)
         stream  = get_stream_for_pad(pad)
         self.current_streams.append(stream)
 
