@@ -264,7 +264,9 @@ class Pipeline(Signallable, Loggable):
         self.debug("state:%r", state)
         res = self._pipeline.set_state(state)
         if res == gst.STATE_CHANGE_FAILURE:
-            raise PipelineError("Failure changing state of the gst.Pipeline")
+            # reset to NULL
+            self._pipeline.set_state(gst.STATE_NULL)
+            raise PipelineError("Failure changing state of the gst.Pipeline to %r, currently reset to NULL" % state)
 
     def getState(self):
         """
@@ -292,6 +294,16 @@ class Pipeline(Signallable, Loggable):
         """
         self.setState(STATE_PAUSED)
 
+        # When the pipeline has been paused we need to update the
+        # timeline/playhead position, as the 'position' signal
+        # is only emitted every 300ms and the playhead jumps
+        # during the playback.
+        try:
+            self.emit("position", self.getPosition())
+        except PipelineError:
+            # Getting the position failed
+            pass
+
     def stop(self):
         """
         Sets the L{Pipeline} to READY
@@ -515,7 +527,7 @@ class Pipeline(Signallable, Loggable):
             return bin_entry.bin
 
         if not automake:
-            raise PipelineError()
+            raise PipelineError("Couldn't get an existing Bin for Factory %r" % factory)
 
         bin = stream_entry.bin = factory.makeBin(stream)
         stream_entry.bin_use_count += 1
@@ -650,7 +662,7 @@ class Pipeline(Signallable, Loggable):
         stream_entry = self._getStreamEntryForFactoryStream(factory, stream)
 
         if stream_entry.tee_use_count == 0:
-            raise PipelineError()
+            raise PipelineError("Tee is already unused from factory %r" % factory)
 
         stream_entry.tee_use_count -= 1
         if stream_entry.tee_use_count == 0:
@@ -740,11 +752,11 @@ class Pipeline(Signallable, Loggable):
         @raise PipelineError: If the Pipeline isn't in NULL or READY.
         """
         if not isinstance(factory, SinkFactory):
-            raise PipelineError()
+            raise PipelineError("Attempting to release a queue for a non-sink factory (%r)" % factory)
 
         stream_entry = self._getStreamEntryForFactoryStream(factory, stream)
         if stream_entry.queue is None:
-            raise PipelineError()
+            raise PipelineError("Couldn't find Queue for Stream/Factory")
 
         stream_entry.queue_use_count -= 1
         if stream_entry.queue_use_count == 0:
@@ -775,6 +787,7 @@ class Pipeline(Signallable, Loggable):
 
     def _busMessageCb(self, unused_bus, message):
         if message.type == gst.MESSAGE_EOS:
+            self.pause()
             self.emit('eos')
         elif message.type == gst.MESSAGE_STATE_CHANGED:
             prev, new, pending = message.parse_state_changed()
@@ -903,7 +916,8 @@ class Pipeline(Signallable, Loggable):
                 action.streamRemoved(factory, stream)
         except:
             self._lock.release()
-        self._lock.release()
+        else:
+            self._lock.release()
 
     def _connectToPadSignals(self, bin):
         # Listen on the given bin for pads being added/removed
