@@ -24,31 +24,22 @@
 Project class
 """
 
-import os.path
-import gst
-import traceback
-from gettext import gettext as _
 from pitivi.log.loggable import Loggable
 from pitivi.timeline.timeline import Timeline
-from pitivi.timeline.track import Track
 from pitivi.stream import AudioStream, VideoStream
 from pitivi.pipeline import Pipeline
 from pitivi.factories.timeline import TimelineSourceFactory
 from pitivi.sourcelist import SourceList
 from pitivi.settings import ExportSettings
-from pitivi.configure import APPNAME
 from pitivi.signalinterface import Signallable
 from pitivi.action import ViewAction
-from pitivi.formatters.format import save_project
+from pitivi.utils import Seeker
+import gst
 
 class ProjectError(Exception):
     """Project error"""
     pass
 
-class ProjectSaveLoadError(ProjectError):
-    """Error while loading/saving project"""
-    pass
-
 class Project(Signallable, Loggable):
     """The base class for PiTiVi projects
 
@@ -70,14 +61,12 @@ class Project(Signallable, Loggable):
     @type loaded: C{bool}
 
     Signals:
-     - C{missing-plugins} : A plugin is missing for the given uri
      - C{loaded} : The project is now fully loaded.
     """
 
     __signals__ = {
-        "settings-changed" : None,
-        "missing-plugins": ["uri", "detail", "description"],
-        "loaded" : None
+        "settings-changed" : ['old', 'new'],
+        "project-changed" : [],
         }
 
     def __init__(self, name="", uri=None, **kwargs):
@@ -93,22 +82,22 @@ class Project(Signallable, Loggable):
         self.uri = uri
         self.urichanged = False
         self.format = None
-        self.sources = SourceList(self)
+        self.sources = SourceList()
+        self.sources.connect("source-added", self._sourceAddedCb)
+        self.sources.connect("source-removed", self._sourceRemovedCb)
 
-        self.settingssigid = 0
         self._dirty = False
 
-        self.sources.connect('missing-plugins', self._sourceListMissingPluginsCb)
-
         self.timeline = Timeline()
 
         self.factory = TimelineSourceFactory(self.timeline)
         self.pipeline = Pipeline()
         self.view_action = ViewAction()
         self.view_action.addProducers(self.factory)
+        self.seeker = Seeker(80)
 
-        # the loading formatter will set this accordingly
-        self.loaded = True
+        settings = self.getSettings()
+        self._videocaps = settings.getVideoCaps()
 
     def release(self):
         self.pipeline.release()
@@ -116,9 +105,6 @@ class Project(Signallable, Loggable):
 
     #{ Settings methods
 
-    def _settingsChangedCb(self, unused_settings):
-        self.emit('settings-changed')
-
     def getSettings(self):
         """
         return the currently configured settings.
@@ -134,16 +120,10 @@ class Project(Signallable, Loggable):
         If settings is None, the current settings will be unset
         """
         self.log("Setting %s as the project's settings", settings)
-        if self.settings:
-            self.settings.disconnect(self.settingssigid)
+        oldsettings = self.settings
         self.settings = settings
-        self.emit('settings-changed')
-        self.settingssigid = self.settings.connect('settings-changed',
-            self._settingsChangedCb)
-
-    def unsetSettings(self, unused_settings):
-        """ Remove the currently configured settings."""
-        self.setSettings(None)
+        self._projectSettingsChanged()
+        self.emit('settings-changed', oldsettings, settings)
 
     def getAutoSettings(self):
         """
@@ -176,9 +156,6 @@ class Project(Signallable, Loggable):
 
     #}
 
-    def _sourceListMissingPluginsCb(self, source_list, uri, detail, description):
-        return self.emit('missing-plugins', uri, detail, description)
-
     #{ Save and Load features
 
     def save(self, location=None, overwrite=False):
@@ -190,15 +167,16 @@ class Project(Signallable, Loggable):
         @type location: C{URI}
         @param overwrite: Whether to overwrite existing location.
         @type overwrite: C{bool}
-
-        @raises ProjectSaveLoadError: If no uri was provided and none was set
-        previously.
         """
+        # import here to break circular import
+        from pitivi.formatters.format import save_project
+        from pitivi.formatters.base import FormatterError
+
         self.log("saving...")
         location = location or self.uri
 
         if location == None:
-            raise ProjectSaveLoadError("Location unknown")
+            raise FormatterError("Location unknown")
 
         save_project(self, location or self.uri, self.format,
                      overwrite)
@@ -207,16 +185,24 @@ class Project(Signallable, Loggable):
 
     def setModificationState(self, state):
         self._dirty = state
+        if state:
+            self.emit('project-changed')
 
     def hasUnsavedModifications(self):
         return self._dirty
 
-    def markLoaded(self):
-        """
-        Mark the project as loaded.
+    def _projectSettingsChanged(self):
+        settings = self.getSettings()
+        self._videocaps = settings.getVideoCaps()
 
-        Will emit the 'loaded' signal. Only meant to be used by
-        L{Formatter}s.
-        """
-        self.loaded = True
-        self.emit('loaded')
+        for fact in self.sources.getSources():
+            fact.setFilterCaps(self._videocaps)
+        if self.pipeline.getState() != gst.STATE_NULL:
+            self.pipeline.stop()
+            self.pipeline.pause()
+
+    def _sourceAddedCb(self, sourcelist, factory):
+        factory.setFilterCaps(self._videocaps)
+
+    def _sourceRemovedCb(self, sourclist, uri, factory):
+        self.timeline.removeFactory(factory)
