@@ -25,6 +25,7 @@ Settings
 """
 
 import os
+import platform
 import gst
 from ConfigParser import SafeConfigParser, ParsingError
 
@@ -37,6 +38,11 @@ from pitivi.encode import available_muxers, available_video_encoders, \
 from pitivi.stream import get_stream_for_caps
 from pitivi.log.loggable import Loggable
 
+if platform.system() == 'Windows':
+    HOME = 'USERPROFILE'
+else:
+    HOME = 'HOME'
+
 def get_bool_env(var):
     value = os.getenv(var)
     if not value:
@@ -51,7 +57,7 @@ def get_bool_env(var):
 
 def get_env_by_type(type_, var):
     """
-    Returns the environement variable.
+    Returns the environment variable.
 
     @arg type_: The type of the variable
     @type type_: C{type}
@@ -60,6 +66,8 @@ def get_env_by_type(type_, var):
     @returns: The content of the environment variable, or C{None} if it doesn't
     exist.
     """
+    if var is None:
+        return None
     if type_ == bool:
         return get_bool_env(var)
     else:
@@ -90,11 +98,15 @@ def get_env_dirs(var, default):
 
 def xdg_config_home(autocreate=True):
     return get_dir(get_env_default("XDG_CONFIG_HOME",
-        os.path.join(os.getenv("HOME"), ".config")), autocreate)
+        os.path.join(os.getenv(HOME), ".config")), autocreate)
 
 def xdg_data_home(autocreate=True):
     return get_dir(get_env_default("XDG_DATA_HOME",
-        os.path.join(os.getenv("HOME"), ".local", "share")), autocreate)
+        os.path.join(os.getenv(HOME), ".local", "share")), autocreate)
+
+def xdg_cache_home(autocreate=True):
+    return get_dir(get_env_default("XDG_CACHE_HOME",
+        os.path.join(os.getenv("HOME"), ".cache")), autocreate)
 
 def xdg_data_dirs():
     return get_env_dirs("XDG_DATA_DIRS",
@@ -469,6 +481,23 @@ class ExportSettings(Signallable, Loggable):
         self.aencoders = available_audio_encoders()
         self.muxers = available_combinations(self.muxers, self.vencoders, self.aencoders)
 
+    def copy(self):
+        ret = ExportSettings()
+        ret.videowidth = self.videowidth
+        ret.videoheight = self.videoheight
+        ret.videorate = gst.Fraction(self.videorate.num, self.videorate.denom)
+        ret.videopar = gst.Fraction(self.videopar.num, self.videopar.denom)
+        ret.audiochannels = self.audiochannels
+        ret.audiorate = self.audiorate
+        ret.audiodepth = self.audiodepth
+        ret.vencoder = self.vencoder
+        ret.aencoder = self.aencoder
+        ret.muxer = self.muxer
+        ret.containersettings = dict(self.containersettings)
+        ret.acodecsettings = dict(self.acodecsettings)
+        ret.vcodecsettings = dict(self.vcodecsettings)
+        return ret
+
     def __str__(self):
         msg = _("Export Settings\n")
         msg += _("Video: ") + str(self.videowidth) + " " + str(self.videoheight) +\
@@ -571,19 +600,20 @@ class ExportSettings(Signallable, Loggable):
         if changed:
             self.emit("encoders-changed")
 
-def export_settings_to_render_settings(export):
+def export_settings_to_render_settings(export,
+        have_video=True, have_audio=True):
     # Get the audio and video caps/encoder/settings
     astream = get_stream_for_caps(export.getAudioCaps())
     vstream = get_stream_for_caps(export.getVideoCaps())
 
     encoder_settings = []
-    if export.vencoder is not None:
+    if export.vencoder is not None and have_video:
         vset = StreamEncodeSettings(encoder=export.vencoder,
                                     input_stream=vstream,
                                     encodersettings=export.vcodecsettings)
         encoder_settings.append(vset)
 
-    if export.aencoder is not None:
+    if export.aencoder is not None and have_audio:
         aset = StreamEncodeSettings(encoder=export.aencoder,
                                     input_stream=astream,
                                     encodersettings=export.acodecsettings)
