@@ -143,7 +143,8 @@ class VideoStream(MultimediaStream):
         self.videotype = struct.get_name()
         self.raw = self.videotype.startswith("video/x-raw-")
 
-        for property_name in ('width', 'height', 'framerate', 'format'):
+        for property_name in ('width', 'height', 'framerate', 'format',
+            'bpp', 'depth'):
             try:
                 setattr(self, property_name, struct[property_name])
             except KeyError:
@@ -165,6 +166,15 @@ class VideoStream(MultimediaStream):
         except:
             self.dar = gst.Fraction(4, 3)
 
+    def has_alpha(self):
+        if self.videotype == "video/x-raw-rgb":
+            return ((hasattr(self, 'bpp') and (self.bpp == 32)) and
+                    (hasattr(self, 'depth') and (self.depth == 32)))
+        elif self.videotype == "video/x-raw-yuv":
+            return (hasattr(self, 'format') and
+                    self.format == gst.Fourcc('AYUV'))
+        return False
+
 class AudioStream(MultimediaStream):
     """
     Audio stream
@@ -330,11 +340,12 @@ def get_stream_for_caps(caps, pad=None):
         ret = VideoStream(caps, pad_name, stream_type == 'image')
     elif stream_type == 'audio':
         ret = AudioStream(caps, pad_name)
-    elif stream_type == 'text':
+    elif stream_type in ('text', 'subpicture'):
         ret = TextStream(caps, pad_name)
     return ret
 
 def get_stream_for_pad(pad):
+    log.debug("stream", "pad:%r")
     caps = pad.props.caps
     if caps is None:
         caps = pad.get_caps()
