@@ -23,12 +23,13 @@ import gst
 
 from pitivi.signalinterface import Signallable
 from pitivi.utils import get_controllable_properties, getPreviousObject, \
-        getNextObject, start_insort_right
+        getNextObject, start_insort_right, between
 from pitivi.log.loggable import Loggable
 from pitivi.stream import VideoStream, AudioStream
 from pitivi.factories.test import VideoTestSourceFactory, \
         AudioTestSourceFactory
-from pitivi.elements.mixer import SmartAdderBin
+from pitivi.elements.mixer import SmartAdderBin, SmartVideomixerBin
+from pitivi.timeline.gap import Gap
 
 class TrackError(Exception):
     pass
@@ -131,14 +132,16 @@ class Interpolator(Signallable, Loggable):
 
     __signals__ = {
         'keyframe-added' : ['keyframe'],
-        'keyframe-removed' : ['keyframe'],
-        'keyframe-moved' : ['keyframe'],
+        'keyframe-removed' : ['keyframe', 'old_value'],
+        'keyframe-moved' : ['keyframe', 'old_value'],
     }
 
-    def __init__(self, trackobject, element, prop, minimum=None, maximum=None):
+    def __init__(self, trackobject, element, prop, minimum=None, maximum=None,
+        format=None):
         Loggable.__init__(self)
         self.debug("track:%r, element:%r, property:%r", trackobject, element, prop)
         self._keyframes = []
+        self.trackobject = trackobject
 
         if minimum is None:
             minimum = prop.minimum
@@ -158,9 +161,15 @@ class Interpolator(Signallable, Loggable):
         self.start = FixedKeyframe(self)
         self.end = FixedKeyframe(self)
         self.start.value = self._default
-        self.start.setObjectTime(0)
+        self.start.setObjectTime(trackobject.in_point)
+        self._keyframeTimeValueChanged(self.start, 0, self.start.value)
         self.end.value = self._default
-        self.end.setObjectTime(trackobject.factory.duration)
+        if trackobject.in_point == trackobject.out_point:
+            self.end.setObjectTime(trackobject.in_point + 1)
+        else:
+            self.end.setObjectTime(trackobject.out_point)
+        self._keyframeTimeValueChanged(self.end, self.end.time, self.end.value)
+        self.format = format if format else str
 
     def attachToElementProperty(self, prop, element):
         self._element = element
@@ -200,6 +209,7 @@ class Interpolator(Signallable, Loggable):
                    gst.TIME_ARGS(keyframe.time), keyframe.value, keyframe.mode)
 
         self._keyframes.append(keyframe)
+        self._keyframes.sort()
 
         self._controller.set(self._property.name, keyframe.time, keyframe.value)
 
@@ -208,10 +218,11 @@ class Interpolator(Signallable, Loggable):
         return keyframe
 
     def removeKeyframe(self, keyframe):
+        old_value = self._controller.get(self._property.name, keyframe.time)
         self._controller.unset(self._property.name, keyframe.time)
         if keyframe is not self.start and keyframe is not self.end:
             self._keyframes.remove(keyframe)
-            self.emit("keyframe-removed", keyframe)
+            self.emit("keyframe-removed", keyframe, old_value)
 
     def setKeyframeMode(self, kf, mode):
         # FIXME: currently InterpolationSourceControllers only support a
@@ -237,25 +248,47 @@ class Interpolator(Signallable, Loggable):
         self.debug("kf.time:%s, ptime:%s, value:%r",
                    gst.TIME_ARGS(kf.time),
                    gst.TIME_ARGS(ptime), value)
+        old_value = self._controller.get(self._property.name, ptime)
+        self._controller.set(self._property.name, ptime, value)
         if kf.time != ptime:
             self._controller.unset(self._property.name, kf.time)
-        self._controller.set(self._property.name, ptime, value)
-        self.emit("keyframe-moved", kf)
+        self.emit("keyframe-moved", kf, old_value)
 
     def getKeyframes(self):
-        # TODO: This could be more efficient. We are re-sorting all the keyframes
-        # every time we iterate over them. One thought would be to keep the
-        # list sorted in-place as keyframes are added and moved.
         yield self.start
-        for kf in sorted(self._keyframes):
+        for kf in self._keyframes:
             yield kf
         yield self.end
 
     def getInteriorKeyframes(self):
         """Same as above but does not include start, or end points"""
-        for kf in sorted(self._keyframes):
+        for kf in self._keyframes:
             yield kf
 
+    def getVisibleKeyframes(self):
+        """Return start, end and any keyframes included in between"""
+        yield self.start
+        start_time = self.start.time
+        end_time = self.end.time
+        for kf in sorted(self._keyframes):
+            if between(start_time, kf.time, end_time):
+                yield kf
+        yield self.end
+
+    def updateMediaStart(self, start):
+        self._keyframeTimeValueChanged(self.start, start, self.start.value)
+        self.start.setObjectTime(start)
+
+    def updateMediaStop(self, stop):
+        self._keyframeTimeValueChanged(self.end, stop, self.end.value)
+        self.end.setObjectTime(stop)
+
+    def valueAt(self, time):
+        return self._controller.get(self._property.name, time)
+
+    def formatValue(self, value):
+        return self.format(value)
+
     keyframes = property(getKeyframes)
 
 class TrackObject(Signallable, Loggable):
@@ -268,6 +301,7 @@ class TrackObject(Signallable, Loggable):
         'media-duration-changed': ['media-duration'],
         'priority-changed': ['priority'],
         'selected-changed' : ['state'],
+        'stagger-changed' : ['stagger'],
     }
 
     def __init__(self, factory, stream, start=0,
@@ -277,10 +311,14 @@ class TrackObject(Signallable, Loggable):
         self.debug("factory:%r", factory)
         self.factory = factory
         self.stream = stream
+        self.stream_type = type(stream)
         self.track = None
         self.timeline_object = None
         self.interpolators = {}
-        self._rebuild_interpolators = True
+        self._rebuild_interpolators = False
+        self._public_priority = priority
+        self._position = 0
+        self._stagger = 0
         self.gnl_object = obj = self._makeGnlObject()
         self.keyframes = []
 
@@ -304,6 +342,7 @@ class TrackObject(Signallable, Loggable):
         obj.props.priority = priority
 
         self._connectToSignals(obj)
+        self._updatePriority(self._public_priority)
 
     def getInterpolator(self, property_name):
         self._maybeBuildInterpolators()
@@ -318,12 +357,12 @@ class TrackObject(Signallable, Loggable):
         return self.interpolators
 
     def _maybeBuildInterpolators(self):
-        if not list(self.gnl_object.elements()):
-            raise TrackError("makeBin hasn't been called yet")
-
         if not self._rebuild_interpolators:
             return
 
+        if not list(self.gnl_object.elements()):
+            raise TrackError("makeBin hasn't been called yet")
+
         self._rebuild_interpolators = False
 
         factory_properties = self.factory.getInterpolatedProperties(self.stream)
@@ -340,19 +379,23 @@ class TrackObject(Signallable, Loggable):
                 interpolator = old_interpolators[prop_name][1]
             except KeyError:
                 if factory_properties[prop_name]:
-                    lower, upper = factory_properties[prop_name]
+                    lower, upper, formatstr = factory_properties[prop_name]
                 else:
-                    lower, upper = None, None
+                    lower, upper, formatstr = None, None, None
                 interpolator = Interpolator(self, gst_object,
-                    gst_object_property, lower, upper)
+                    gst_object_property, lower, upper, formatstr)
             else:
-                interpolator.attachToElementProperty(gst_object_property,
-                        gst_object)
-
                 # remove and add again the keyframes so they are set on the
                 # current controller
-                for keyframe in list(interpolator.keyframes):
+                keyframes = list(interpolator.keyframes)
+                for keyframe in keyframes:
                     interpolator.removeKeyframe(keyframe)
+
+                interpolator.attachToElementProperty(gst_object_property,
+                        gst_object)
+                interpolator.updateMediaStop(self.out_point)
+
+                for keyframe in keyframes:
                     interpolator.newKeyframe(keyframe)
 
             self.interpolators[gst_object_property.name] = \
@@ -467,7 +510,7 @@ class TrackObject(Signallable, Loggable):
     rate = property(_getRate)
 
     def _getPriority(self):
-        return self.gnl_object.props.priority
+        return self._public_priority
 
     def setPriority(self, priority):
         if self.timeline_object is not None:
@@ -476,11 +519,24 @@ class TrackObject(Signallable, Loggable):
             self.setObjectPriority(priority)
 
     def setObjectPriority(self, priority):
-        if self.gnl_object.props.priority != priority:
-            self.gnl_object.props.priority = priority
+        if priority != self._public_priority:
+            self._updatePriority(priority)
+
+    def _updatePriority(self, priority):
+        if self.stream_type is VideoStream:
+            true_priority = 2 + self._stagger + (3 * priority)
+        elif self.stream_type is AudioStream:
+            true_priority  = 2 + (2 * self._stagger) + (4 * priority)
+        if self.gnl_object.props.priority != true_priority:
+            self.gnl_object.props.priority = true_priority
 
     priority = property(_getPriority, setPriority)
 
+    def _getStagger(self):
+        return self._stagger
+
+    stagger = property(_getStagger)
+
     def trimStart(self, position, snap=False):
         if self.timeline_object is not None:
             self.timeline_object.trimStart(position, snap)
@@ -515,10 +571,18 @@ class TrackObject(Signallable, Loggable):
 
         other = self.copy()
 
+        # update interpolators
+        for prop, i in self.interpolators.itervalues():
+            value = i.valueAt(position)
+            i.end.setValue(value)
+
+        for prop, i in other.interpolators.itervalues():
+            value = i.valueAt(position)
+            i.start.setValue(value)
+
         other.trimObjectStart(position)
         self.setObjectDuration(position - self.gnl_object.props.start)
         self.setObjectMediaDuration(position - self.gnl_object.props.start)
-
         return other
 
     # True when the track object is part of the timeline's current selection
@@ -543,6 +607,7 @@ class TrackObject(Signallable, Loggable):
 
         bin = self.factory.makeBin(self.stream)
         self.gnl_object.add(bin)
+        self._rebuild_interpolators = True
         self._maybeBuildInterpolators()
 
     def releaseBin(self):
@@ -559,16 +624,30 @@ class TrackObject(Signallable, Loggable):
         self.emit('duration-changed', obj.props.duration)
 
     def _notifyMediaStartCb(self, obj, pspec):
-        self.emit('in-point-changed', obj.props.media_start)
+        start = obj.props.media_start
+        self.emit('in-point-changed', start)
+        for p, i in self.interpolators.itervalues():
+            i.updateMediaStart(start)
 
     def _notifyMediaDurationCb(self, obj, pspec):
         self.emit('media-duration-changed', obj.props.media_duration)
 
     def _notifyMediaStopCb(self, obj, pspec):
-        self.emit('out-point-changed', obj.props.media_stop)
+        stop = obj.props.media_stop
+        self.emit('out-point-changed', stop)
+        for p, i in self.interpolators.itervalues():
+            i.updateMediaStop(stop)
 
     def _notifyPriorityCb(self, obj, pspec):
-        self.emit('priority-changed', obj.props.priority)
+        if self.stream_type is VideoStream:
+            true_priority = obj.props.priority
+            public_priority = (true_priority - 2 - self._stagger) // 3
+        elif self.stream_type is AudioStream:
+            true_priority = obj.props.priority
+            public_priority = (true_priority - 2 - (2 * self._stagger))// 4
+        if self._public_priority != public_priority:
+            self._public_priority = public_priority
+            self.emit('priority-changed', public_priority)
 
     def _connectToSignals(self, gnl_object):
         gnl_object.connect('notify::start', self._notifyStartCb)
@@ -593,45 +672,243 @@ class TrackObject(Signallable, Loggable):
     def _makeGnlObject(self):
         raise NotImplementedError()
 
+    def updatePosition(self, position):
+        if position != self._position:
+            self._position = position
+            self._stagger = position & 1
+            self._updatePriority(self._public_priority)
+            self.emit("stagger-changed", self._stagger)
+
 class SourceTrackObject(TrackObject):
+
+    numobjs = 0
+
     def _makeGnlObject(self):
-        source = gst.element_factory_make('gnlsource')
+        source = gst.element_factory_make('gnlsource', 
+            "gnlsource: " + self.factory.__class__.__name__ +
+            str(SourceTrackObject.numobjs))
+        SourceTrackObject.numobjs += 1
         return source
 
+class Transition(Signallable):
+
+    __signals__ = {
+        "start-changed" : ["start"],
+        "duration-changed" : ["duration"],
+        "priority-changed" : ["priority"],
+    }
+
+    def __init__(self, a, b):
+        self.a = a
+        self.b = b
+        self.start = 0
+        self.duration = 0
+        self.priority = 0
+        self._makeGnlObject()
+        self._connectToTrackObjects(a, b)
+
+    def _makeGnlObject(self):
+        pass
+
+    def _connectToTrackObjects(self, a, b):
+        a.connect("start-changed", self._updateStartDuration)
+        a.connect("duration-changed", self._updateStartDuration)
+        a.connect("priority-changed", self._updatePriority)
+        b.connect("start-changed", self._updateStartDuration)
+        b.connect("priority-changed", self._updatePriority)
+        a.connect("stagger-changed", self._staggerChanged)
+        b.connect("stagger-changed", self._staggerChanged)
+        self._updateStartDuration()
+        self._updatePriority()
+
+    def _updateStartDuration(self, *unused):
+        start = self.b.start
+        end = self.a.start + self.a.duration
+        duration = max(0, end - start)
+
+        if start != self.start:
+            self._updateOperationStart(start)
+            self.start = start
+            self.emit("start-changed", start)
+
+        if duration != self.duration:
+            self._updateOperationDuration(duration)
+            self.duration = duration
+            self.emit("duration-changed", duration)
+
+        self._updateController()
+
+    def _updatePriority(self, *unused):
+        if self.a.priority == self.b.priority:
+            priority = self.a.priority
+            self._updateOperationPriority(priority)
+            self.priority = priority
+            self.emit("priority-changed", priority)
+
+    def _staggerChanged(self, *unused):
+        self._updateController()
+
+    def _updateController(self):
+        pass
+
+    def _updateOperationStart(self, start):
+        pass
+
+    def _updateOperationDuration(self, duration):
+        pass
+
+    def _updateOperationPriority(self, priority):
+        pass
+
+    def addThyselfToComposition(self, composition):
+        pass
+
+    def removeThyselfFromComposition(self, composition):
+        pass
+
+class VideoTransition(Transition):
+
+    caps = gst.Caps("video/x-raw-yuv,format=(fourcc)AYUV")
+
+    def _makeGnlObject(self):
+        trans = gst.element_factory_make("alpha")
+        self.controller = gst.Controller(trans, "alpha")
+        self.controller.set_interpolation_mode("alpha", gst.INTERPOLATE_LINEAR)
+
+        self.operation = gst.element_factory_make("gnloperation")
+        self.operation.add(trans)
+        self.operation.props.media_start = 0
+        self.operation.props.caps = self.caps
+
+    def addThyselfToComposition(self, composition):
+        composition.add(self.operation)
+
+    def removeThyselfFromComposition(self, composition):
+        composition.remove(self.operation)
+        self.operation.set_state(gst.STATE_NULL)
+
+    def _updateOperationStart(self, start):
+        self.operation.props.start = start
+
+    def _updateOperationDuration(self, duration):
+        self.operation.props.duration = duration
+        self.operation.props.media_duration = duration
+
+    def _updateOperationPriority(self, priority):
+        self.operation.props.priority = 1 + 3 * priority
+
+    def _updateController(self):
+        if self.a.stagger > self.b.stagger:
+            # source a is under source b (higher priority)
+            # we fade source B in
+            self.controller.unset_all("alpha")
+            self.controller.set("alpha", 0, 0.0)
+            self.controller.set("alpha", self.duration, 1.0)
+        elif self.a.stagger < self.b.stagger:
+            # source a is over source b (lower priority)
+            # we fade source a out
+            self.controller.unset_all("alpha")
+            self.controller.set("alpha", 0, 1.0)
+            self.controller.set("alpha", self.duration, 0.0)
+
+class AudioTransition(Transition):
+
+    def _makeGnlObject(self):
+        trans = gst.element_factory_make("volume")
+        self.a_controller = gst.Controller(trans, "volume")
+        self.a_controller.set_interpolation_mode("volume", gst.INTERPOLATE_LINEAR)
+
+        self.a_operation = gst.element_factory_make("gnloperation")
+        self.a_operation.add(trans)
+
+        trans = gst.element_factory_make("volume")
+        self.b_controller = gst.Controller(trans, "volume")
+        self.b_controller.set_interpolation_mode("volume", gst.INTERPOLATE_LINEAR)
+
+        self.b_operation = gst.element_factory_make("gnloperation")
+        self.b_operation.add(trans)
+
+        self.a_operation.props.media_start = 0
+
+        self.b_operation.props.media_start = 0
+
+
+    def addThyselfToComposition(self, composition):
+        composition.add(self.a_operation, self.b_operation)
+
+    def removeThyselfFromComposition(self, composition):
+        composition.remove(self.a_operation)
+        composition.remove(self.b_operation)
+        self.a_operation.set_state(gst.STATE_NULL)
+        self.b_operation.set_state(gst.STATE_NULL)
+
+    def _updateOperationStart(self, start):
+        self.a_operation.props.start = start
+        self.b_operation.props.start = start
+
+    def _updateOperationDuration(self, duration):
+        self.a_operation.props.duration = duration
+        self.a_operation.props.media_duration = duration
+        self.b_operation.props.duration = duration
+        self.b_operation.props.media_duration = duration
+
+    def _staggerChanged(self, *unused):
+        self._updateOperationPriority(self.priority)
+
+    def _updateOperationPriority(self, priority):
+        self.a_operation.props.priority = 1 + 2 * self.a.stagger + 4 * priority
+        self.b_operation.props.priority = 1 + 2 * self.b.stagger + 4 * priority
+
+    def _updateController(self):
+        self.a_controller.set("volume", 0, 1.0)
+        self.a_controller.set("volume", self.duration, 0.0)
+        self.b_controller.set("volume", 0, 0.0)
+        self.b_controller.set("volume", self.duration, 1.0)
+
+class Track(Signallable, Loggable):
+    logCategory = "track"
 
-class Track(Signallable):
     __signals__ = {
         'start-changed': ['start'],
         'duration-changed': ['duration'],
         'track-object-added': ['track_object'],
         'track-object-removed': ['track_object'],
-        'max-priority-changed': ['track_object']
+        'max-priority-changed': ['track_object'],
+        'transition-added' : ['transition'],
+        'transition-removed' : ['transition'],
     }
 
     def __init__(self, stream):
         self.stream = stream
+        if type(self.stream) is VideoStream:
+            self.TransitionClass = VideoTransition
+        elif type(self.stream) is AudioStream:
+            self.TransitionClass = AudioTransition
         self.composition = gst.element_factory_make('gnlcomposition')
         self.composition.connect('notify::start', self._compositionStartChangedCb)
         self.composition.connect('notify::duration', self._compositionDurationChangedCb)
         self.track_objects = []
-        self.default_track_object = None
+        self.transitions = {}
+        self._update_transitions = True
         self._max_priority = 0
 
-        default_track_object = self._getDefaultTrackObjectForStream(stream)
-        if default_track_object:
-            self.setDefaultTrackObject(default_track_object)
-
         self.mixer = self._getMixerForStream(stream)
         if self.mixer:
             self.composition.add(self.mixer)
+        self.default_sources = []
 
     def _getDefaultTrackObjectForStream(self, stream):
         if isinstance(stream, VideoStream):
-            return self._getDefaultVideoTrackObject(stream)
+            ret = self._getDefaultVideoTrackObject(stream)
         elif isinstance(stream, AudioStream):
-            return self._getDefaultAudioTrackObject(stream)
+            ret = self._getDefaultAudioTrackObject(stream)
+        else:
+            return None
 
-        return None
+        ret.makeBin()
+        ret.gnl_object.props.priority = 2 ** 32 - 1
+
+        return ret
 
     def _getDefaultVideoTrackObject(self, stream):
         factory = VideoTestSourceFactory(pattern='black')
@@ -645,6 +922,54 @@ class Track(Signallable):
 
         return track_object
 
+    def _defaultSourceBlockedCb(self, pad, blocked):
+        pass
+
+    def _shutdownDefaultSource(self, source):
+        source = list(source.elements())[0]
+        for srcpad in source.src_pads():
+            srcpad = source.get_pad('src')
+            srcpad.set_blocked_async(True, self._defaultSourceBlockedCb)
+            srcpad.push_event(gst.event_new_flush_start())
+
+    def _sourceDebug(self, source):
+        t = gst.TIME_ARGS
+        return "%s [%s - %s]" % (source, t(source.props.start),
+                t(source.props.start + source.props.duration))
+
+    def _updateDefaultSourcesUnchecked(self):
+        for source in self.default_sources:
+            self.debug("removing default source %s", self._sourceDebug(source))
+            self._shutdownDefaultSource(source)
+            self.composition.remove(source)
+            source.set_state(gst.STATE_NULL)
+        gaps = Gap.findAllGaps(self.track_objects)
+
+        self.default_sources = []
+        for gap in gaps:
+            source = self._getDefaultTrackObjectForStream(self.stream)
+            gnl_object = source.gnl_object
+            gnl_object.props.start = gap.start
+            gnl_object.props.duration = gap.initial_duration
+            self.debug("adding default source %s",
+                    self._sourceDebug(source.gnl_object))
+            self.composition.add(gnl_object)
+            self.default_sources.append(gnl_object)
+
+    def updateDefaultSources(self):
+        if not self.composition.props.update:
+            return
+
+        self.updateDefaultSourcesReal()
+
+    def updateDefaultSourcesReal(self):
+        update = self.composition.props.update
+        self.composition.props.update = True
+        try:
+            self._updateDefaultSourcesUnchecked()
+        finally:
+            self.composition.props.update = update
+
     def _getMixerForStream(self, stream):
         if isinstance(stream, AudioStream):
             gnl = gst.element_factory_make("gnloperation", "top-level-audio-mixer")
@@ -653,38 +978,34 @@ class Track(Signallable):
             gnl.props.expandable = True
             gnl.props.priority = 0
             return gnl
+        elif isinstance(stream, VideoStream):
+            gnl = gst.element_factory_make("gnloperation", "top-level-video-mixer")
+            m = SmartVideomixerBin(self)
+            gnl.add(m)
+            gnl.props.expandable = True
+            gnl.props.priority = 0
+            gnl.connect("input-priority-changed",
+                        self._videoInputPriorityChangedCb, m)
+            return gnl
         return None
 
+    def _videoInputPriorityChangedCb(self, operation, pad, priority, mixer):
+        self.debug("operation %s pad %s priority changed %s",
+                operation, pad, priority)
+        mixer.update_priority(pad, priority)
+
     def _getStart(self):
         return self.composition.props.start
 
-    def setDefaultTrackObject(self, track_object):
-        if self.default_track_object is not None:
-            self.removeTrackObject(self.default_track_object)
-
-        self.default_track_object = None
-        track_object.gnl_object.props.priority = 2**32-1
-        self.default_track_object = track_object
-        try:
-            self.addTrackObject(track_object)
-        except:
-            self.default_track_object = None
-            raise
-
-    def _skipDefaultTrackObject(self, timeline_object):
-        return timeline_object is self.default_track_object
-
     def getPreviousTrackObject(self, obj, priority=-1):
-        prev = getPreviousObject(obj, self.track_objects, priority,
-                self._skipDefaultTrackObject)
+        prev = getPreviousObject(obj, self.track_objects, priority)
         if prev is None:
             raise TrackError("no previous track object", obj)
 
         return prev
 
     def getNextTrackObject(self, obj, priority=-1):
-        next = getNextObject(obj, self.track_objects, priority,
-                self._skipDefaultTrackObject)
+        next = getNextObject(obj, self.track_objects, priority)
         if next is None:
             raise TrackError("no next track object", obj)
 
@@ -702,16 +1023,9 @@ class Track(Signallable):
 
     max_priority = property(_getMaxPriority)
 
-    _max_priority = 0
-
-    @property
-    def max_priority(self):
-        return self._max_priority
-
     def _trackObjectPriorityCb(self, trackobject, priority):
         op = self._max_priority
-        self._max_priority = max((obj.priority for obj in self.track_objects
-            if obj is not self.default_track_object))
+        self._max_priority = max((obj.priority for obj in self.track_objects))
         if op != self._max_priority:
             self.emit("max-priority-changed", self._max_priority)
 
@@ -732,6 +1046,7 @@ class Track(Signallable):
         track_object.track = self
 
         start_insort_right(self.track_objects, track_object)
+        self.updateDefaultSources()
 
         try:
             self.composition.add(track_object.gnl_object)
@@ -744,6 +1059,8 @@ class Track(Signallable):
         self._connectToTrackObject(track_object)
 
         self.emit('track-object-added', track_object)
+        if self._update_transitions:
+            self.updateTransitions()
 
     def removeTrackObject(self, track_object):
         if track_object.track is None:
@@ -760,11 +1077,15 @@ class Track(Signallable):
 
         self.track_objects.remove(track_object)
         track_object.track = None
+
         self._disconnectTrackObjectSignals(track_object)
 
         self._updateMaxPriority()
+        self.updateDefaultSources()
 
         self.emit('track-object-removed', track_object)
+        if self._update_transitions:
+            self.updateTransitions()
 
     def removeAllTrackObjects(self):
         for track_object in list(self.track_objects):
@@ -772,7 +1093,7 @@ class Track(Signallable):
 
     def _updateMaxPriority(self):
         priorities = [track_object.priority for track_object in
-            self.track_objects if track_object is not self.default_track_object]
+            self.track_objects]
         if not priorities:
             max_priority = 0
         else:
@@ -814,6 +1135,100 @@ class Track(Signallable):
 
     def enableUpdates(self):
         self.composition.props.update = True
+        self.updateDefaultSources()
+        self._update_transitions = True
+        self.updateTransitions()
 
     def disableUpdates(self):
         self.composition.props.update = False
+        self._update_transitions = False
+
+    def addTransition(self, transition):
+        a, b = transition.a, transition.b
+        if not ((a in self.track_objects) and
+                (b in self.track_objects)):
+            raise TrackError("One or both track objects not in track")
+        if (a, b) in self.transitions:
+            raise TrackError(
+                "A transition is already defined for these objects")
+        transition.addThyselfToComposition(self.composition)
+        self.transitions[a, b] = transition
+        self.emit("transition-added", transition)
+
+    def removeTransition(self, transition):
+        a, b = transition.a, transition.b
+        transition.removeThyselfFromComposition(self.composition)
+        del self.transitions[a, b]
+        self.emit("transition-removed", transition)
+
+    def getTrackObjectsGroupedByLayer(self):
+        layers = [[] for x in xrange(0, self.max_priority + 1)]
+        for track_object in self.track_objects:
+            layers[int(track_object.priority)].append(track_object)
+        return layers
+
+    def getValidTransitionSlots(self, objs):
+        prev = None
+        safe = 0
+        duration = 0
+        slots = []
+        valid = True
+
+        def pop():
+            if len(slots):
+                slots.pop(-1)
+        for obj in objs:
+            end = obj.start + obj.duration
+            if obj.start >= duration:
+                safe = obj.start
+                duration = end
+                prev = obj
+            elif end >= duration and obj.start >= safe:
+                slots.append((prev, obj))
+                safe = duration
+                duration = end
+                prev = obj
+            elif end >= duration and obj.start < safe:
+                pop()
+                valid = False
+                safe = duration
+                duration = end
+                prev = obj
+            elif end < duration and obj.start >= safe:
+                safe = end
+                valid = False
+            elif end < duration and obj.start < safe:
+                pop()
+                valid = False
+                safe = end
+        return slots, valid
+
+    valid_arrangement = True
+
+    def updateTransitions(self):
+        # create all new transitions
+        valid_slots = set()
+        all_valid = True
+        for layer in self.getTrackObjectsGroupedByLayer():
+            pos = 0
+            prev = None
+            slots, is_valid = self.getValidTransitionSlots(layer)
+            all_valid &= is_valid
+            for slot in slots:
+                a, b = slot
+                if a == prev:
+                    b.updatePosition(pos)
+                    pos += 1
+                else:
+                    a.updatePosition(pos)
+                    b.updatePosition(pos + 1)
+                    pos += 2
+                prev = b
+                valid_slots.add(slot)
+                if not slot in self.transitions:
+                    tr = self.TransitionClass(a, b)
+                    self.addTransition(tr)
+        current_slots = set(self.transitions.iterkeys())
+        for slot in current_slots - valid_slots:
+            self.removeTransition(self.transitions[slot])
+        self.valid_arrangement = all_valid
