@@ -10,8 +10,8 @@ from gettext import ngettext
 GlobalSettings.addConfigSection("user-interface")
 LAYER_HEIGHT_EXPANDED = 50
 LAYER_HEIGHT_COLLAPSED = 15
-LAYER_SPACING = 5
-TRACK_SPACING = 5
+LAYER_SPACING = 15
+TRACK_SPACING = 8
 
 def pack_color_32(red, green, blue, alpha = 0xFFFF):
    red = red >> 8
@@ -150,3 +150,23 @@ def beautify_stream(stream):
 
     raise NotImplementedError
 
+# from http://cairographics.org/cookbook/roundedrectangles/
+def roundedrec(context,x,y,w,h,r = 10):
+    "Draw a rounded rectangle"
+    #   A****BQ
+    #  H      C
+    #  *      *
+    #  G      D
+    #   F****E
+
+    context.move_to(x+r,y)                      # Move to A
+    context.line_to(x+w-r,y)                    # Straight line to B
+    context.curve_to(x+w,y,x+w,y,x+w,y+r)       # Curve to C, Control points are both at Q
+    context.line_to(x+w,y+h-r)                  # Move to D
+    context.curve_to(x+w,y+h,x+w,y+h,x+w-r,y+h) # Curve to E
+    context.line_to(x+r,y+h)                    # Line to F
+    context.curve_to(x,y+h,x,y+h,x,y+h-r)       # Curve to G
+    context.line_to(x,y+r)                      # Line to H
+    context.curve_to(x,y,x,y,x+r,y)             # Curve to A
+    return
+
