@@ -31,22 +31,28 @@ from pitivi.ui.zoominterface import Zoomable
 import pitivi.ui.previewer as previewer
 from pitivi.ui.view import View
 from pitivi.ui.controller import Controller
-from pitivi.ui.common import LAYER_HEIGHT_EXPANDED
+from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, roundedrec
 import pitivi.ui.point as point
-
-def between(a, b, c):
-    return (a <= b) and (b <= c)
+from pitivi.utils import between
 
 def intersect(b1, b2):
     return goocanvas.Bounds(max(b1.x1, b2.x1), max(b1.y1, b2.y1),
         min(b1.x2, b2.x2), min(b1.y2, b2.y2))
 
-KW_WIDTH = 7
-KW_HEIGHT = 7
+KW_WIDTH = 10
+KW_HEIGHT = 10
 KW_WIDTH2 = KW_WIDTH / 2
 KW_HEIGHT2 = KW_HEIGHT / 2
 KW_MOUSE_WIDTH = KW_WIDTH2 + 1
 KW_MOUSE_HEIGHT = KW_HEIGHT2 + 1
+KW_LABEL_X_OFFSET = KW_WIDTH2 + 5
+KW_LABEL_Y_OFFSET = -10
+KW_LABEL_X_OVERFLOW = 50
+KW_LABEL_Y_OVERFLOW = 12
+KW_LABEL_HPAD = 6
+KW_LABEL_VPAD = 4
+KW_LABEL_HPAD2 = KW_LABEL_VPAD / 2
+KW_LABEL_VPAD2 = KW_LABEL_VPAD / 2
 CURVE_STROKE_WIDTH = 2.0
 HAND = gtk.gdk.Cursor(gtk.gdk.HAND2)
 
@@ -58,6 +64,7 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
 
         _cursor = HAND
         _kf = None
+        _handle_enter_leave = False
 
         def drag_start(self, item, target, event):
             self._view.app.action_log.begin("volume change")
@@ -111,8 +118,9 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
             bounds = view.bounds
             time = (Zoomable.pixelToNs(pos[0] - bounds.x1) +
                 view.element.in_point)
-            value = ((1 - (pos[1] - bounds.y1 - view._min) / view._range) *
-                interpolator.range) + interpolator.lower
+            value = ((1 - (pos[1] - KW_LABEL_Y_OVERFLOW - bounds.y1 - 
+                view._min) / view._range) * 
+                    interpolator.range) + interpolator.lower
             return time, value
 
         def enter(self, item ,target):
@@ -137,6 +145,7 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
         self.interpolator = interpolator
         self._focused_kf = None
         self.normal()
+        self.set_simple_transform(0, -KW_LABEL_Y_OVERFLOW, 1.0, 0)
 
 ## properties
 
@@ -152,9 +161,7 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
 
 ## element callbacks
 
-    def _set_element(self):
-        self.previewer = previewer.get_preview_for_object(self.element)
-    element = receiver(setter=_set_element)
+    element = receiver()
 
     @handler(element, "in-point-changed")
     @handler(element, "media-duration-changed")
@@ -166,7 +173,7 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
     interpolator = receiver()
 
     @handler(interpolator, "keyframe-removed")
-    def keyframeRemoved(self, unused_interpolator, keyframe):
+    def keyframeRemoved(self, unused_interpolator, keyframe, old_value=None):
         if keyframe in self.keyframes:
             del self.keyframes[keyframe]
             if keyframe is self._focused_kf:
@@ -174,10 +181,13 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
         self.changed(False)
 
     @handler(interpolator, "keyframe-added")
-    @handler(interpolator, "keyframe-moved")
     def curveChanged(self, unused_interpolator, unused_keyframe):
         self.changed(False)
 
+    @handler(interpolator, "keyframe-moved")
+    def curveChanged(self, unused_interpolator, unused_keyframe, old_value=None):
+        self.changed(False)
+
 ## Zoomable interface overries
 
     def zoomChanged(self):
@@ -188,14 +198,16 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
     def do_simple_update(self, cr):
         cr.identity_matrix()
         if self.element.factory:
+            self.visible_width = self.nsToPixel(self.element.duration)
             self.bounds = goocanvas.Bounds(0, 0,
-            Zoomable.nsToPixel(self.element.duration), self.height)
+                self.visible_width + KW_LABEL_X_OVERFLOW,
+                self.height + KW_LABEL_Y_OVERFLOW)
 
     def _getKeyframeXY(self, kf):
         interp = self.interpolator
         x = self.nsToPixel(kf.time - self.element.in_point)
-        y = self._range - (((kf.value - interp.lower) / interp.range) *
-            self._range)
+        y = KW_LABEL_Y_OVERFLOW + self._range - (((kf.value - 
+            interp.lower) / interp.range) * self._range)
         return point.Point(x + self.bounds.x1, y + self.bounds.y1 + self._min)
 
     def _controlPoint(self, cr, kf):
@@ -205,13 +217,20 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
         self.keyframes[kf] = pos
 
     def do_simple_paint(self, cr, bounds):
-        bounds = intersect(self.bounds, bounds)
+        cr.save()
         cr.identity_matrix()
         if self.interpolator:
-            height = bounds.y2 - bounds.y1
-            width = bounds.x2 - bounds.x1
-            cr.rectangle(bounds.x1, bounds.y1, width, height)
+
+            # set clipping region to the visible portion of the clip
+            vis_bounds = intersect(
+                goocanvas.Bounds(
+                    self.bounds.x1, self.bounds.y1 + KW_LABEL_Y_OVERFLOW,
+                    self.bounds.x2 - KW_LABEL_X_OVERFLOW, self.bounds.y2), bounds)
+            vis_width = vis_bounds.x2 - vis_bounds.x1
+            vis_height = vis_bounds.y2 - vis_bounds.y1
+            cr.rectangle(vis_bounds.x1, vis_bounds.y1, vis_width, vis_height)
             cr.clip()
+
             self.make_curve(cr)
             cr.set_line_width(self.line_width)
             cr.set_source_rgb(1, 0, 0)
@@ -222,33 +241,64 @@ class Curve(goocanvas.ItemSimple, goocanvas.Item, View, Zoomable):
             cr.fill_preserve()
             cr.set_source_rgb(1, 0, 0)
             cr.stroke()
+
             # re-draw the focused keyframe, if it exists, inverted
             if self._focused_kf:
+
                 self._controlPoint(cr, self._focused_kf)
                 cr.set_source_rgb(1, 0, 0)
                 cr.fill_preserve()
                 cr.set_source_rgb(1, 1, 1)
                 cr.stroke()
 
+                x, y = self.keyframes[self._focused_kf]
+                x += KW_LABEL_X_OFFSET
+                y += KW_LABEL_Y_OFFSET
+                text = self.interpolator.formatValue(self._focused_kf.value)
+                w, h = cr.text_extents("0" * len(text))[2:4]
+
+                # reset clip region to the full bounds of the item
+                bounds = intersect(self.bounds, bounds)
+                width = bounds.x2 - bounds.x1
+                height = bounds.y2 - bounds.y1
+                # cr.reset_clip()
+                cr.restore()
+                cr.rectangle(bounds.x1, bounds.y1, width, height)
+                cr.clip()
+
+                # draw the value label
+                roundedrec(cr, x - KW_LABEL_HPAD2, y - KW_LABEL_VPAD2, 
+                    w + KW_LABEL_HPAD, h + KW_LABEL_VPAD, r=10)
+                cr.set_source_rgb(1, 1, 1)
+                cr.fill()
+                cr.set_source_rgb(1, 0, 0)
+                cr.move_to(x, y + h)
+                cr.show_text(text)
+            else:
+                cr.restore()
+
     def make_curve(self, cr):
         if not self.interpolator:
             return
-        iterator = self.interpolator.keyframes
+        iterator = self.interpolator.getVisibleKeyframes()
         cr.move_to(*self._getKeyframeXY(iterator.next()))
         for kf in iterator:
             cr.line_to(*self._getKeyframeXY(kf))
         cr.line_to(*self._getKeyframeXY(self.interpolator.end))
 
     def make_keyframes(self, cr):
-        for kf in self.interpolator.keyframes:
+        for kf in self.interpolator.getVisibleKeyframes():
             self._controlPoint(cr, kf)
 
     def do_simple_is_item_at(self, x, y, cr, pointer_event):
-        if (between(0, x, self.nsToPixel(self.element.duration)) and
-            between(0, y, self.height)):
+        if (between(0, x, self.visible_width) and
+            between(KW_LABEL_Y_OVERFLOW, y, self.height + 
+            KW_LABEL_Y_OVERFLOW)):
             x += self.bounds.x1
             y += self.bounds.y1
+            cr.new_path()
             self.make_curve(cr)
+            self.make_keyframes(cr)
             cr.set_line_width(10.0)
             return cr.in_stroke(x, y) or bool(self.findKeyframe((x, y)))
         return False
