@@ -27,6 +27,8 @@ import os
 import time
 import gtk
 import gst
+from urlparse import urlparse
+import urllib
 
 from gettext import gettext as _
 
@@ -36,8 +38,9 @@ from pitivi.ui.exportsettingswidget import ExportSettingsDialog
 from pitivi.ui.glade import GladeWindow
 from pitivi.action import render_action_for_uri, ViewAction
 from pitivi.factories.base import SourceFactory
+from pitivi.factories.timeline import TimelineSourceFactory
 from pitivi.settings import export_settings_to_render_settings
-from pitivi.stream import AudioStream, VideoStream
+from pitivi.stream import VideoStream, AudioStream
 from pitivi.utils import beautify_length
 
 class EncodingDialog(GladeWindow, Loggable):
@@ -45,8 +48,8 @@ class EncodingDialog(GladeWindow, Loggable):
     glade_file = "encodingdialog.glade"
 
     def __init__(self, app, project, pipeline=None):
-        GladeWindow.__init__(self)
         Loggable.__init__(self)
+        GladeWindow.__init__(self)
 
         self.app = app
 
@@ -67,6 +70,8 @@ class EncodingDialog(GladeWindow, Loggable):
             self.pipeline = pipeline
         else:
             self.pipeline = self.project.pipeline
+        self.detectStreamTypes()
+
         self.outfile = None
         self.rendering = False
         self.renderaction = None
@@ -83,8 +88,15 @@ class EncodingDialog(GladeWindow, Loggable):
         self.destroy()
 
     def _displaySettings(self):
-        self.vinfo.set_markup(self.settings.getVideoDescription())
-        self.ainfo.set_markup(self.settings.getAudioDescription())
+        if self.have_video:
+            self.vinfo.set_markup(self.settings.getVideoDescription())
+        else:
+            self.vinfo.set_markup("no video")
+
+        if self.have_audio:
+            self.ainfo.set_markup(self.settings.getAudioDescription())
+        else:
+            self.ainfo.set_markup("no audio")
 
     def _fileButtonClickedCb(self, button):
         dialog = gtk.FileChooserDialog(title=_("Choose file to render to"),
@@ -92,9 +104,11 @@ class EncodingDialog(GladeWindow, Loggable):
                                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                                 gtk.STOCK_OK, gtk.RESPONSE_ACCEPT),
                                        action=gtk.FILE_CHOOSER_ACTION_SAVE)
+        dialog.set_icon_name("pitivi")
         if self.outfile:
-            dialog.set_uri(self.outfile)
-            dialog.set_current_name(os.path.basename(self.outfile))
+            fullfilename = urlparse(self.outfile).path
+            dialog.set_filename(urllib.url2pathname(fullfilename))
+            dialog.set_current_name(urllib.url2pathname(os.path.basename(fullfilename)))
         else:
             dialog.set_current_folder(self.app.settings.lastExportFolder)
 
@@ -102,7 +116,8 @@ class EncodingDialog(GladeWindow, Loggable):
         dialog.hide()
         if res == gtk.RESPONSE_ACCEPT:
             self.outfile = dialog.get_uri()
-            button.set_label(os.path.basename(self.outfile))
+            shortfilename = os.path.basename(urlparse(self.outfile).path)
+            button.set_label(urllib.url2pathname(shortfilename))
             self.recordbutton.set_sensitive(True)
             self.progressbar.set_text("")
             self.app.settings.lastExportFolder = dialog.get_current_folder()
@@ -121,6 +136,11 @@ class EncodingDialog(GladeWindow, Loggable):
             if length:
                 self.progressbar.set_text(_("About %s left") % length)
 
+    def _changeSourceSettings(self, settings):
+        videocaps = settings.getVideoCaps()
+        for source in self.project.sources.getSources():
+            source.setFilterCaps(videocaps)
+
     def _recordButtonClickedCb(self, unused_button):
         self.debug("Rendering")
         if self.outfile and not self.rendering:
@@ -162,6 +182,23 @@ class EncodingDialog(GladeWindow, Loggable):
         self.debug("delete event")
         self._shutDown()
 
+    def detectStreamTypes(self):
+        self.have_video = False
+        self.have_audio = False
+
+        # we can only render TimelineSourceFactory
+        sources = [factory for factory in self.pipeline.factories.keys()
+                if isinstance(factory, SourceFactory)]
+        timeline_source = sources[0]
+        assert isinstance(timeline_source, TimelineSourceFactory)
+
+        for track in timeline_source.timeline.tracks:
+            if isinstance(track.stream, AudioStream) and track.duration > 0:
+                self.have_audio = True
+            elif isinstance(track.stream, VideoStream) and \
+                    track.duration > 0:
+                self.have_video = True
+
     def addRecordAction(self):
         self.debug("renderaction %r", self.renderaction)
         if self.renderaction == None:
@@ -169,10 +206,13 @@ class EncodingDialog(GladeWindow, Loggable):
             self.pipeline.connect('eos', self._eosCb)
             self.debug("Setting pipeline to STOP")
             self.pipeline.stop()
-            settings = export_settings_to_render_settings(self.settings)
-            sources = [x for x in self.pipeline.factories if isinstance(x, SourceFactory)]
+            settings = export_settings_to_render_settings(self.settings,
+                    self.have_video, self.have_audio)
             self.debug("Creating RenderAction")
-            self.renderaction = render_action_for_uri(self.outfile, settings, *sources)
+            sources = [factory for factory in self.pipeline.factories
+                    if isinstance(factory, SourceFactory)]
+            self.renderaction = render_action_for_uri(self.outfile,
+                    settings, *sources)
             self.debug("setting action on pipeline")
             self.pipeline.addAction(self.renderaction)
             self.debug("Activating render action")
@@ -181,6 +221,8 @@ class EncodingDialog(GladeWindow, Loggable):
             for ac in self.pipeline.actions:
                 if isinstance(ac, ViewAction) and ac.isActive():
                     ac.setSync(False)
+            self.debug("Updating all sources to render settings")
+            self._changeSourceSettings(self.settings)
             self.debug("setting pipeline to PAUSE")
             self.pipeline.pause()
             self.debug("done")
@@ -195,6 +237,7 @@ class EncodingDialog(GladeWindow, Loggable):
             for ac in self.pipeline.actions:
                 if isinstance(ac, ViewAction) and ac.isActive():
                     ac.setSync(True)
+            self._changeSourceSettings(self.project.getSettings())
             self.pipeline.pause()
             self.pipeline.disconnect_by_function(self._positionCb)
             self.pipeline.disconnect_by_function(self._eosCb)
