@@ -24,6 +24,7 @@ Main GTK+ window
 """
 
 import os
+import platform
 import gtk
 import gobject
 gobject.threads_init()
@@ -46,7 +47,7 @@ from pitivi.ui.timeline import Timeline
 from pitivi.ui.projecttabs import ProjectTabs
 from pitivi.ui.viewer import PitiviViewer
 from pitivi.configure import pitivi_version, APPNAME, get_pixmap_dir, \
-     get_global_pixmap_dir
+     get_global_pixmap_dir, LIBDIR
 from pitivi.ui import dnd
 from pitivi.pipeline import Pipeline
 from pitivi.action import ViewAction
@@ -126,6 +127,7 @@ def create_stock_icons():
     gtk.stock_add([
             ('pitivi-render', _('Render'), 0, 0, 'pitivi'),
             ('pitivi-split', _('Split'), 0, 0, 'pitivi'),
+            ('pitivi-keyframe', _('Keyframe'), 0, 0, 'pitivi'),
             ('pitivi-unlink', _('Unlink'), 0, 0, 'pitivi'),
             # Translators: This is an action, the title of a button
             ('pitivi-link', _('Link'), 0, 0, 'pitivi'),
@@ -136,10 +138,11 @@ def create_stock_icons():
     pixmaps = {
         "pitivi-render" : "pitivi-render-24.png",
         "pitivi-split" : "pitivi-split-24.svg",
+        "pitivi-keyframe" : "pitivi-keyframe-24.svg",
         "pitivi-unlink" : "pitivi-unlink-24.svg",
         "pitivi-link" : "pitivi-relink-24.svg",
-        "pitivi-ungroup" : "pitivi-unlink-24.svg",
-        "pitivi-group" : "pitivi-relink-24.svg",
+        "pitivi-ungroup" : "pitivi-ungroup-24.svg",
+        "pitivi-group" : "pitivi-group-24.svg",
     }
     factory = gtk.IconFactory()
     pmdir = get_pixmap_dir()
@@ -191,6 +194,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 self._projectManagerProjectSavedCb)
         self.app.projectManager.connect("closing-project",
                 self._projectManagerClosingProjectCb)
+        self.app.projectManager.connect("reverting-to-saved",
+                self._projectManagerRevertingToSavedCb)
         self.app.projectManager.connect("project-closed",
                 self._projectManagerProjectClosedCb)
         self.app.projectManager.connect("missing-uri",
@@ -203,10 +208,12 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         # if no webcams available, hide the webcam action
         if self.app.deviceprobe is not None:
-            self.app.deviceprobe.connect("device-added", self._deviceChangeCb)
-            self.app.deviceprobe.connect("device-removed", self._deviceChangeCb)
-            if len(self.app.deviceprobe.getVideoSourceDevices()) < 1:
-                self.webcam_button.set_sensitive(False)
+            # On Windows disable device probe
+            if platform.system() != 'Windows':
+                self.app.deviceprobe.connect("device-added", self._deviceChangeCb)
+                self.app.deviceprobe.connect("device-removed", self._deviceChangeCb)
+                if len(self.app.deviceprobe.getVideoSourceDevices()) < 1:
+                    self.webcam_button.set_sensitive(False)
         else:
             self.webcam_button.set_sensitive(False)
 
@@ -251,6 +258,8 @@ class PitiviMainWindow(gtk.Window, Loggable):
              None, _("Save the current project"), self._saveProjectCb),
             ("SaveProjectAs", gtk.STOCK_SAVE_AS, None,
              None, _("Save the current project"), self._saveProjectAsCb),
+            ("RevertToSavedProject", gtk.STOCK_REVERT_TO_SAVED, None,
+             None, _("Reload the current project"), self._revertToSavedProjectCb),             
             ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project Settings"),
              None, _("Edit the project settings"), self._projectSettingsCb),
             ("RenderProject", 'pitivi-render' , _("_Render project"),
@@ -355,12 +364,16 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.uimanager = gtk.UIManager()
         self.add_accel_group(self.uimanager.get_accel_group())
         self.uimanager.insert_action_group(self.actiongroup, 0)
+        if 'pitivi.exe' in __file__.lower():
+            xml = LIBDIR + '\\pitivi.exe'
+        else:
+            xml = __file__
         self.uimanager.add_ui_from_file(os.path.join(os.path.dirname(
-            os.path.abspath(__file__)), "mainwindow.xml"))
+            os.path.abspath(xml)), "mainwindow.xml"))
 
     def _createUi(self, instance):
         """ Create the graphical interface """
-        self.set_title("%s v%s" % (APPNAME, pitivi_version))
+        self.set_title("%s" % (APPNAME))
         self.connect("delete-event", self._deleteCb)
         self.connect("configure-event", self._configureCb)
 
@@ -390,7 +403,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.projecttabs = ProjectTabs()
 
         self.sourcelist = SourceList(instance, self.uimanager)
-        self.projecttabs.append_page(self.sourcelist, gtk.Label(_("Clip Library")))
+        self.projecttabs.append_page(self.sourcelist, gtk.Label(_("Media Library")))
         self._connectToSourceList()
         self.sourcelist.show()
 
@@ -450,6 +463,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         #pulseaudio 'role' (http://0pointer.de/blog/projects/tagging-audio.htm
         os.environ["PULSE_PROP_media.role"] = "production"
+        os.environ["PULSE_PROP_application.icon_name"] = "pitivi"
 
     def _connectToSourceList(self):
         self.sourcelist.connect('play', self._sourceListPlayCb)
@@ -571,6 +585,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         return False
 
+    def _revertToSavedProjectCb(self, unused_action):
+        return self.app.projectManager.revertToSavedProject()
+            
+
     def _projectSettingsCb(self, unused_action):
         from projectsettings import ProjectSettingsDialog
         ProjectSettingsDialog(self, self.app.current).show()
@@ -689,7 +707,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def loop(self, unused_action):
         pass
 
-
     def _projectManagerNewProjectLoadedCb(self, projectManager, project):
         self.log("A NEW project is loaded, update the UI!")
         self.project = project
@@ -808,6 +825,30 @@ class PitiviMainWindow(gtk.Window, Loggable):
         project.seeker.disconnect_by_func(self._timelineSeekCb)
         return False
 
+    def _projectManagerRevertingToSavedCb(self, projectManager, project):
+        if project.hasUnsavedModifications():
+            dialog = gtk.MessageDialog(self,
+                                gtk.DIALOG_MODAL,
+                                gtk.MESSAGE_WARNING,
+                                gtk.BUTTONS_NONE,
+                                _("Do you want to reload current project?")
+                                )
+            dialog.set_icon_name("pitivi")
+            dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_NO,
+                                        gtk.STOCK_REVERT_TO_SAVED, gtk.RESPONSE_YES)
+            dialog.set_title(_("Revert to saved project"))
+            dialog.set_resizable(False)
+            dialog.set_property("secondary-text", 
+                                            _("All unsaved changes will be lost.")
+                                        )
+            dialog.set_default_response(gtk.RESPONSE_NO)
+            response = dialog.run()
+            dialog.destroy()
+            if response <> gtk.RESPONSE_YES:
+                return False
+        return True
+        
+
     def _projectManagerNewProjectFailedCb(self, projectManager, uri, exception):
         # ungrey UI
         dialog = gtk.MessageDialog(self,
@@ -895,6 +936,9 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dirty = action_log.dirty()
         save_action = self.actiongroup.get_action("SaveProject")
         save_action.set_sensitive(dirty)
+        if self.app.current.uri is not None:
+            revert_action = self.actiongroup.get_action("RevertToSavedProject")
+            revert_action.set_sensitive(dirty)
         self.app.current.setModificationState(dirty)
 
         redo_action = self.actiongroup.get_action("Redo")
@@ -902,7 +946,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         redo_action.set_sensitive(can_redo)
 
         if self.project is not None:
-            app_name = "%s %s" % (APPNAME, pitivi_version)
+            app_name = "%s" % (APPNAME)
             title = u"%s \u2014 %s" % (self.project.name, app_name)
             if dirty:
                 title = "*" + title
@@ -943,10 +987,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
     @handler(project_pipeline, "error")
     def _pipelineErrorCb(self, unused_pipeline, error, detail):
-        if detail:
-            self.timeline.infostub.addErrors(error, detail)
-        else:
-            self.timeline.infostub.addErrors(error)
+        pass
 
     @handler(project_pipeline, "position")
     def _timelinePipelinePositionChangedCb(self, pipeline, position):
@@ -1071,4 +1112,3 @@ class PitiviMainWindow(gtk.Window, Loggable):
             self.project.pipeline.seek(position, format)
         except:
             self.debug("Seeking failed")
-
