@@ -37,7 +37,6 @@ from pitivi.elements.arraysink import ArraySink
 from pitivi.signalinterface import Signallable
 import pitivi.stream as stream
 from pitivi.settings import GlobalSettings
-import pitivi.instance as instance
 from pitivi.ui.zoominterface import Zoomable
 from pitivi.log.loggable import Loggable
 from pitivi.factories.file import PictureFileSourceFactory
@@ -108,7 +107,7 @@ PreferencesDialog.addTogglePreference('showWaveforms',
 
 previewers = {}
 
-def get_preview_for_object(trackobject):
+def get_preview_for_object(instance, trackobject):
     factory = trackobject.factory
     for stream_ in factory.getOutputStreams():
         if stream_.isCompatible(trackobject.track.stream):
@@ -124,14 +123,14 @@ def get_preview_for_object(trackobject):
         # note that we switch on the stream_type, but we hash on the stream
         # itself.
         if stream_type == stream.AudioStream:
-            previewers[key] = RandomAccessAudioPreviewer(factory, stream_)
+            previewers[key] = RandomAccessAudioPreviewer(instance, factory, stream_)
         elif stream_type == stream.VideoStream:
             if type(factory) == PictureFileSourceFactory:
-                previewers[key] = StillImagePreviewer(factory, stream_)
+                previewers[key] = StillImagePreviewer(instance, factory, stream_)
             else:
-                previewers[key] = RandomAccessVideoPreviewer(factory, stream_)
+                previewers[key] = RandomAccessVideoPreviewer(instance, factory, stream_)
         else:
-            previewers[key] = DefaultPreviewer(factory, stream_)
+            previewers[key] = DefaultPreviewer(instance, factory, stream_)
     return previewers[key]
 
 class Previewer(Signallable, Loggable):
@@ -148,12 +147,12 @@ class Previewer(Signallable, Loggable):
 
     aspect = 4.0 / 3.0
 
-    def __init__(self, factory, stream_):
+    def __init__(self, instance, factory, stream_):
         Loggable.__init__(self)
         # create default thumbnail
         path = os.path.join(get_pixmap_dir(), self.__DEFAULT_THUMB__)
         self.default_thumb = cairo.ImageSurface.create_from_png(path)
-        self._connectSettings(instance.PiTiVi.settings)
+        self._connectSettings(instance.settings)
 
     def render_cairo(self, cr, bounds, element, y1):
         """Render a preview of element onto a cairo context within the current
@@ -182,9 +181,9 @@ class RandomAccessPreviewer(Previewer):
     the stream, and time segments. This allows the UI to re-draw the affected
     portion of a thumbnail sequence or audio waveform."""
 
-    def __init__(self, factory, stream_):
+    def __init__(self, instance, factory, stream_):
         self._view = True
-        Previewer.__init__(self, factory, stream_)
+        Previewer.__init__(self, instance, factory, stream_)
         self._queue = []
 
         # FIXME:
@@ -196,7 +195,6 @@ class RandomAccessPreviewer(Previewer):
 
         # assume 50 pixel height
         self.theight = 50
-        self.twidth = int(self.aspect * self.theight)
         self.waiting_timestamp = None
 
         self._pipelineInit(factory, bin)
@@ -230,7 +228,6 @@ class RandomAccessPreviewer(Previewer):
 
         # tdur = duration in ns of thumbnail
         # sof  = start of file in pixel coordinates
-        tdur = Zoomable.pixelToNs(self.twidth + self._spacing())
         x1 = bounds.x1;
         sof = Zoomable.nsToPixel(element.start - element.in_point)
 
@@ -252,12 +249,14 @@ class RandomAccessPreviewer(Previewer):
         # further, which would result in fewer thumbnails needing to be
         # generated.
         j = Zoomable.pixelToNs(i - sof)
+        istep = self.twidth + self._spacing()
+        jstep = self.tdur + Zoomable.pixelToNs(self.spacing)
 
         while i < bounds.x2:
-            cr.set_source_surface(self._thumbForTime(j), i, y1)
+            self._thumbForTime(cr, j, i, y1)
             cr.rectangle(i - 1, y1, self.twidth + 2, self.theight)
-            i += self.twidth + self._spacing()
-            j += tdur
+            i += istep
+            j += jstep
             cr.fill()
 
     def _spacing(self):
@@ -272,13 +271,14 @@ class RandomAccessPreviewer(Previewer):
 
         raise NotImplementedError
 
-    def _thumbForTime(self, time):
+    def _thumbForTime(self, cr, time, x, y):
         segment = self._segment_for_time(time)
         if segment in self._cache:
-            return self._cache[segment]
-
-        self._requestThumbnail(segment)
-        return self.default_thumb
+            surface = self._cache[segment]
+        else:
+            self._requestThumbnail(segment)
+            surface = self.default_thumb
+        cr.set_source_surface(surface, x, y)
 
     def _finishThumbnail(self, surface, segment):
         """Notifies the preview object that the a new thumbnail is ready to be
@@ -304,7 +304,9 @@ class RandomAccessPreviewer(Previewer):
         the next thumbnail in the queue. This should always be called from the
         main application thread."""
         if self._queue:
-            self._startThumbnail(self._queue[0])
+            if not self._startThumbnail(self._queue[0]):
+                self._queue.pop(0)
+                self._nextThumbnail()
         return False
 
     def _requestThumbnail(self, segment):
@@ -342,10 +344,23 @@ class RandomAccessPreviewer(Previewer):
 
 class RandomAccessVideoPreviewer(RandomAccessPreviewer):
 
-    def __init__(self, factory, stream_):
+    @property
+    def twidth(self):
+        return int(self.aspect * self.theight)
+
+    @property
+    def tdur(self):
+        return Zoomable.pixelToNs(self.twidth)
+
+    def __init__(self, instance, factory, stream_):
         if stream_.dar and stream_.par:
             self.aspect = float(stream_.dar)
-        RandomAccessPreviewer.__init__(self, factory, stream_)
+        rate = stream_.framerate
+        RandomAccessPreviewer.__init__(self, instance, factory, stream_)
+        self.tstep = Zoomable.pixelToNsAt(self.twidth, Zoomable.max_zoom)
+        if rate.num:
+            frame_duration = (gst.SECOND * rate.denom) / rate.num
+            self.tstep = max(frame_duration, self.tstep)
 
     def _pipelineInit(self, factory, sbin):
         csp = gst.element_factory_make("ffmpegcolorspace")
@@ -366,16 +381,15 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
         self.videopipeline.set_state(gst.STATE_PAUSED)
 
     def _segment_for_time(self, time):
-        # for video thumbnails, the duration doesn't matter
-        return time
+        # quantize thumbnail timestamps to maximum granularity
+        return time - (time % self.tstep)
 
     def _thumbnailCb(self, unused_thsink, pixbuf, timestamp):
         gobject.idle_add(self._finishThumbnail, pixbuf, timestamp)
 
     def _startThumbnail(self, timestamp):
         RandomAccessPreviewer._startThumbnail(self, timestamp)
-        self.log("timestamp : %s", gst.TIME_ARGS(timestamp))
-        self.videopipeline.seek(1.0,
+        return self.videopipeline.seek(1.0,
             gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
             gst.SEEK_TYPE_SET, timestamp,
             gst.SEEK_TYPE_NONE, -1)
@@ -391,11 +405,20 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
 
 class StillImagePreviewer(RandomAccessVideoPreviewer):
 
-    def _thumbForTime(self, time):
-        return RandomAccessVideoPreviewer._thumbForTime(self, 0L)
+    def _thumbForTime(self, cr, time, x, y):
+        return RandomAccessVideoPreviewer._thumbForTime(self, cr, 0L, x, y)
 
 class RandomAccessAudioPreviewer(RandomAccessPreviewer):
 
+    def __init__(self, instance, factory, stream_):
+        self.tdur = 30 * gst.SECOND
+        self.base_width = int(Zoomable.max_zoom)
+        RandomAccessPreviewer.__init__(self, instance, factory, stream_)
+
+    @property
+    def twidth(self):
+        return Zoomable.nsToPixel(self.tdur)
+
     def _pipelineInit(self, factory, sbin):
         self.spacing = 0
 
@@ -406,7 +429,10 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
             conv : self.audioSink,
             self.audioSink : None})
         bus = self.audioPipeline.get_bus()
-        bus.set_sync_handler(self._bus_message)
+        bus.add_signal_watch()
+        bus.connect("message::segment-done", self._busMessageSegmentDoneCb)
+        bus.connect("message::error", self._busMessageErrorCb)
+
         self._audio_cur = None
         self.audioPipeline.set_state(gst.STATE_PAUSED)
 
@@ -415,67 +441,121 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
 
     def _segment_for_time(self, time):
         # for audio files, we need to know the duration the segment spans
-        return time, Zoomable.pixelToNs(self.twidth)
+        return time - (time % self.tdur), self.tdur
 
-    def _bus_message(self, bus, message):
-        if message.type == gst.MESSAGE_SEGMENT_DONE:
-            self._finishWaveform()
+    def _busMessageSegmentDoneCb(self, bus, message):
+        self.debug("segment done")
+        self._finishWaveform()
 
-        elif message.type == gst.MESSAGE_ERROR:
-            error, debug = message.parse_error()
-            # FIXME: do something intelligent here
-            print "Event bus error:", str(error), str(debug)
+    def _busMessageErrorCb(self, bus, message):
+        error, debug = message.parse_error()
+        print "Event bus error:", str(error), str(debug)
 
         return gst.BUS_PASS
 
     def _startThumbnail(self, (timestamp, duration)):
         RandomAccessPreviewer._startThumbnail(self, (timestamp, duration))
         self._audio_cur = timestamp, duration
-        self.audioPipeline.seek(1.0,
+        res = self.audioPipeline.seek(1.0,
             gst.FORMAT_TIME,
             gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE | gst.SEEK_FLAG_SEGMENT,
             gst.SEEK_TYPE_SET, timestamp,
             gst.SEEK_TYPE_SET, timestamp + duration)
+        if not res:
+            self.warning("seek failed %s", timestamp)
         self.audioPipeline.set_state(gst.STATE_PLAYING)
 
+        return res
+
     def _finishWaveform(self):
+        surfaces = []
         surface = cairo.ImageSurface(cairo.FORMAT_A8,
-            int(self.twidth) + 2, self.theight)
+            self.base_width, self.theight)
         cr = cairo.Context(surface)
-        self._plotWaveform(cr, self.audioSink.samples)
+        self._plotWaveform(cr, self.base_width)
         self.audioSink.reset()
-        gobject.idle_add(self._finishThumbnail, surface, self._audio_cur)
 
-    def _plotWaveform(self, cr, samples):
-        hscale = 25
-        if not samples:
-            cr.move_to(0, hscale)
-            cr.line_to(self.twidth, hscale)
-            cr.stroke()
-            return
+        for width in [25, 100, 200]:
+            scaled = cairo.ImageSurface(cairo.FORMAT_A8,
+               width, self.theight)
+            cr = cairo.Context(scaled)
+            matrix = cairo.Matrix()
+            matrix.scale(self.base_width/width, 1.0)
+            cr.set_source_surface(surface)
+            cr.get_source().set_matrix(matrix)
+            cr.rectangle(0, 0, width, self.theight)
+            cr.fill()
+            surfaces.append(scaled)
+        surfaces.append(surface)
+        gobject.idle_add(self._finishThumbnail, surfaces, self._audio_cur)
 
+    def _plotWaveform(self, cr, base_width):
         # clear background
         cr.set_source_rgba(1, 1, 1, 0.0)
-        cr.rectangle(0, 0, self.twidth, self.theight)
+        cr.rectangle(0, 0, base_width, self.theight)
         cr.fill()
 
-        # resample for improved speed
-        stride = 100
-        samples = [max(samples[i:i + stride]) for i in
-            xrange(0, len(samples), stride)]
-        scale = float(self.twidth) / len(samples)
+        samples = self.audioSink.samples
 
-        # generate points
+        if not samples:
+            return
+
+        # find the samples-per-pixel ratio
+        spp = len(samples) / base_width
+        if spp == 0:
+            spp = 1
+        channels = self.audioSink.channels
+        stride = spp * channels
+        hscale = self.theight / (2 * channels)
+
+        # plot points from min to max over a given hunk
+        chan = 0
+        y = hscale
+        while chan < channels:
+            i = chan
+            x = 0
+            while i < len(samples):
+                slice = samples[i:i + stride:channels]
+                min_ = min(slice)
+                max_ = max(slice)
+                cr.move_to(x, y - (min_ * hscale))
+                cr.line_to(x, y - (max_ * hscale))
+                i += spp
+                x += 1
+            y += 2 * hscale
+            chan += 1
+
+        # Draw!
         cr.set_source_rgba(0, 0, 0, 1.0)
-
-        cr.move_to(0, hscale)
-        for x, y in enumerate(samples):
-            x = x * scale
-            y = hscale - (y * hscale)
-            cr.line_to(x, y)
-
         cr.stroke()
 
+    def _thumbForTime(self, cr, time, x, y):
+        segment = self._segment_for_time(time)
+        twidth = self.twidth
+        if segment in self._cache:
+            surfaces = self._cache[segment]
+            if twidth > 200:
+                surface = surfaces[3]
+                base_width = self.base_width
+            elif twidth <= 200:
+                surface = surfaces[2]
+                base_width = 200
+            elif twidth <= 100:
+                surface = surfaces[1]
+                base_width = 100
+            elif twidth <= 25:
+                surface = surfaces[0]
+                base_width = 25
+            x_scale = float(base_width) / self.twidth
+            cr.set_source_surface(surface)
+            matrix = cairo.Matrix()
+            matrix.scale(x_scale, 1.0)
+            matrix.translate(-x, -y)
+            cr.get_source().set_matrix(matrix)
+        else:
+            self._requestThumbnail(segment)
+            cr.set_source_rgba(0.0, 0.0, 0.0, 0.0)
+
     def _connectSettings(self, settings):
         RandomAccessPreviewer._connectSettings(self, settings)
         self._view = settings.showWaveforms
