@@ -20,8 +20,10 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+import platform
 import gobject
 import gtk
+from gtk import gdk
 import gst
 
 from pitivi.action import ViewAction
@@ -170,7 +172,6 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.action = action
         # FIXME: fix this properly?
         self.drawingarea.action = action
-        self.drawingarea.have_set_xid = False
         dar = float(4/3)
         try:
             producer = action.producers[0]
@@ -211,7 +212,6 @@ class PitiviViewer(gtk.VBox, Loggable):
                                       obey_child=False)
         self.pack_start(self.aframe, expand=True)
         self.drawingarea = ViewerWidget(self.action)
-        self.drawingarea.connect_after("expose-event", self._drawingAreaExposeCb)
         self.aframe.add(self.drawingarea)
 
         # Slider
@@ -263,13 +263,27 @@ class PitiviViewer(gtk.VBox, Loggable):
         bbox.pack_start(self.timelabel, expand=False, padding=10)
         self._haveUI = True
 
-        # show the controls and force the aspect frame to have at least the same
-        # width (+110, which is a magic number to minimize dead padding).
-        bbox.show_all()
-        width, height = bbox.size_request()
-        width += 110
-        height = width / self.aframe.props.ratio
-        self.aframe.set_size_request(width , height)
+        screen = gdk.screen_get_default()
+        height = screen.get_height()
+        if height >= 800:
+            # show the controls and force the aspect frame to have at least the same
+            # width (+110, which is a magic number to minimize dead padding).
+            bbox.show_all()
+            width, height = bbox.size_request()
+            width += 110
+            height = int(width / self.aframe.props.ratio)
+            self.aframe.set_size_request(width , height)
+        self.show_all()
+
+    _showingSlider = True
+
+    def showSlider(self):
+        self._showingSlider = True
+        self.slider.show()
+
+    def hideSlider(self):
+        self._showingSlider = False
+        self.slider.hide()
 
     def showControls(self):
         if not self.action:
@@ -280,7 +294,8 @@ class PitiviViewer(gtk.VBox, Loggable):
             self.playpause_button.show()
             self.next_button.show()
             self.forward_button.show()
-            self.slider.show()
+            if self._showingSlider:
+                self.slider.show()
         else:
             self.rewind_button.hide()
             self.back_button.hide()
@@ -302,30 +317,20 @@ class PitiviViewer(gtk.VBox, Loggable):
         except:
             self.warning("could not set ratio !")
 
-    def _drawingAreaExposeCb(self, drawingarea, event):
-        drawingarea.disconnect_by_func(self._drawingAreaExposeCb)
-        for state in range(gtk.STATE_INSENSITIVE + 1):
-            drawingarea.modify_bg(state, drawingarea.style.black)
-        self.debug("yay, we are exposed !")
-        if self.pipeline:
-            try:
-                self.pipeline.paused()
-            except:
-                self.currentState = gst.STATE_NULL
-            else:
-                self.currentState = gst.STATE_PAUSED
-        return False
-
     ## gtk.HScale callbacks for self.slider
 
-    def _sliderButtonPressCb(self, slider, unused_event):
+    def _sliderButtonPressCb(self, slider, event):
+        # borrow totem hack for seek-on-click behavior
+        event.button = 2
         self.info("button pressed")
         self.moving_slider = True
         self.valuechangedid = slider.connect("value-changed", self._sliderValueChangedCb)
         self.pipeline.pause()
         return False
 
-    def _sliderButtonReleaseCb(self, slider, unused_event):
+    def _sliderButtonReleaseCb(self, slider, event):
+        # borrow totem hack for seek-on-click behavior
+        event.button = 2
         self.info("slider button release at %s", time_to_string(long(slider.get_value())))
         self.moving_slider = False
         if self.valuechangedid:
@@ -450,7 +455,8 @@ class PitiviViewer(gtk.VBox, Loggable):
         name = message.structure.get_name()
         self.log('message:%s / %s', message, name)
         if name == 'prepare-xwindow-id':
-            self.drawingarea.set_xwindow_id()
+            sink = message.src
+            sink.set_xwindow_id(self.drawingarea.window_xid)
 
 
 class ViewerWidget(gtk.DrawingArea, Loggable):
@@ -464,18 +470,16 @@ class ViewerWidget(gtk.DrawingArea, Loggable):
         gtk.DrawingArea.__init__(self)
         Loggable.__init__(self)
         self.action = action # FIXME : Check if it's a view action
-        self.have_set_xid = False
         self.unset_flags(gtk.SENSITIVE)
+        for state in range(gtk.STATE_INSENSITIVE + 1):
+            self.modify_bg(state, self.style.black)
 
-    def set_xwindow_id(self):
-        """ set the widget's XID on the configured videosink. """
-        self.log("...")
-        if self.have_set_xid:
-            return
-        self.action.set_window_xid(self.window.xid)
-        self.unset_flags(gtk.DOUBLE_BUFFERED)
-        self.have_set_xid = True
-
+    def do_realize(self):
+        gtk.DrawingArea.do_realize(self)
+        if platform.system() == 'Windows':
+            self.window_xid = self.window.handle
+        else:
+            self.window_xid  = self.window.xid
 
 class PlayPauseButton(gtk.Button, Loggable):
     """ Double state gtk.Button which displays play/pause """
@@ -489,6 +493,7 @@ class PlayPauseButton(gtk.Button, Loggable):
     def __init__(self):
         gtk.Button.__init__(self, label="")
         Loggable.__init__(self)
+        self.get_settings().props.gtk_button_images = True
         self.playing = True
         self.setPlay()
         self.connect('clicked', self._clickedCb)
