@@ -58,14 +58,28 @@ class TestCase(unittest.TestCase):
         del self._tracked
 
     def setUp(self):
+        self._num_failures = len(getattr(self._result, 'failures', []))
+        self._num_errors = len(getattr(self._result, 'errors', []))
         if detect_leaks:
             self.gctrack()
 
     def tearDown(self):
+        # don't barf gc info all over the console if we have already failed a
+        # test case
+        if (self._num_failures < len(getattr(self._result, 'failures', []))
+            or self._num_errors < len(getattr(self._result, 'failures', []))):
+            return
         if detect_leaks:
             self.gccollect()
             self.gcverify()
 
+    # override run() to save a reference to the test result object
+    def run(self, result=None):
+        if not result:
+            result = self.defaultTestResult()
+        self._result = result
+        unittest.TestCase.run(self, result)
+
 # Some fake factories
 class FakeSourceFactory(SourceFactory):
     def __init__(self, factoryname="fakesrc", *args, **kwargs):
