@@ -3,7 +3,7 @@ import os
 import sys
 import unittest
 
-SKIP_FILES = ['common', 'runtests', 'testcomplex', 'testHList', 'testmagets']
+SKIP_FILES = ['common', 'runtests', 'test_integration']
 
 def gettestnames(which):
     if not which:
@@ -19,14 +19,23 @@ def gettestnames(which):
 suite = unittest.TestSuite()
 loader = unittest.TestLoader()
 
-for name in gettestnames(sys.argv[1:]):
-    suite.addTest(loader.loadTestsFromName(name))
+TEST_CASE=os.getenv("TESTCASE")
+
+if TEST_CASE:
+    suite.addTest(loader.loadTestsFromName(TEST_CASE))
+    if not suite._tests:
+        raise Exception("could not find test case %r" % TEST_CASE)
+else:
+    for name in gettestnames(sys.argv[1:]):
+        suite.addTest(loader.loadTestsFromName(name))
 
 descriptions = 1
 verbosity = 1
 if os.environ.has_key('VERBOSE'):
     descriptions = 2
     verbosity = 2
+from pitivi.log import log
+log.init('PITIVI_DEBUG', 1)
 
 testRunner = unittest.TextTestRunner(descriptions=descriptions,
     verbosity=verbosity)
