@@ -120,12 +120,40 @@ class Configuration(object):
         for source in sources:
             self.addSource(*source)
 
+    def clone(self):
+        ret = Configuration()
+        for source in self.sources:
+            if len(source) == 3:
+                name, uri, props = source
+                ret.addSource(name, uri, dict(props))
+            if len(source) == 2:
+                ret.addBadSource(*source)
+        return ret
+
     def addSource(self, name, uri, props=None, error=False):
         if name in self.source_map:
             raise Exception("Duplicate source: '%d' already defined" % name)
         self.sources.append((name, uri, props))
         self.source_map[name] = uri, props
 
+    def updateSource(self, name, uri=None, props=None):
+        def findSource(name):
+            for i, source in enumerate(self.sources):
+                if source[0] == name:
+                    return i
+            raise Exception("Source %s not in configuration" %
+                name)
+
+        i = findSource(name)
+        name, orig_uri, orig_props = self.sources[i]
+        if not uri:
+            uri = orig_uri
+        if props:
+            orig_props.update(props)
+
+        self.sources[i] = (name, uri, orig_props)
+        self.source_map[name] = (uri, orig_props)
+
     def addBadSource(self, name, uri):
         if name in self.source_map:
             raise Exception("Duplicate source: '%d' already defined" % name)
@@ -235,6 +263,16 @@ class InstanceRunner(Signallable):
         container = self.container()
         setattr(self, attrname, container)
         self.tracks[track] = container 
+        container.transitions = {}
+        track.connect("transition-added", self._transitionAddedCb, container)
+        track.connect("transition-removed", self._transitionRemovedCb,
+            container)
+
+    def _transitionAddedCb(self, track, transition, container):
+        container.transitions[(transition.a, transition.b)] = transition
+
+    def _transitionRemovedCb(self, track, transition, container):
+        del container.transitions[(transition.a, transition.b)]
 
     def _setupTimeline(self, configuration):
         for name, uri, props in configuration:
@@ -325,6 +363,12 @@ class Base(TestCase):
     Uses a WatchDog to ensure that test cases will eventually terminate with an
     assertion failure if runtime errors occur inside the mainloop."""
 
+    def run(self, result):
+        self._result = result
+        self._num_failures = len(result.failures)
+        self._num_errors = len(result.errors)
+        TestCase.run(self, result)
+
     def setUp(self):
         TestCase.setUp(self)
         ptv = InteractivePitivi()
@@ -345,9 +389,17 @@ class Base(TestCase):
         # make sure we aren't exiting because our watchdog activated
         self.assertFalse(self.runner.watchdog.activated)
         # make sure the instance has been unset
-        self.assertEquals(pitivi.instance.PiTiVi, None)
+        will_fail = False
+        if ((self._num_errors == self._result.errors) and
+            (self._num_failures == self._result.failures)):
+            will_fail = not (pitivi.instance.PiTiVi is None)
+
+        pitivi.instance.PiTiVi = None
         del self.ptv
         del self.runner
+
+        if will_fail:
+            raise Exception("Instance was not unset")
         TestCase.tearDown(self)
 
 class TestBasic(Base):
@@ -721,5 +773,320 @@ class TestRippleExtensive(Base):
         self.scrub_func = rippleMoveComplexRandomScrubFunc
         self.runner.run()
 
+class TestTransitions(Base):
+
+    def testSimple(self):
+        initial = Configuration()
+        initial.addSource(
+            "object1", 
+            test1, 
+            {
+                "start" : 0,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object2", 
+            test1,
+            {
+                "start" : 5 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object3",
+            test1,
+            {
+                "start" : 10 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+
+        moves = [
+            (9 * gst.SECOND, 0),
+            (1 * gst.SECOND, 0),
+        ]
+
+        expected = [
+            ("object2", "object3", 10 * gst.SECOND, 4 * gst.SECOND, 0),
+            ("object1", "object2", 1 * gst.SECOND, 4 * gst.SECOND, 0),
+        ]
+
+        def timelineConfigured(runner):
+            nextMove()
+
+        def nextMove():
+            if moves:
+                self._cur_move = moves.pop(0)
+                context = MoveContext(self.runner.timeline, 
+                    self.runner.video1.object2,
+                        set([self.runner.video1.object2]))
+                brush.scrub(context, self._cur_move[0], self._cur_move[1], steps=10)
+            else:
+                self.runner.shutDown()
+
+        def scrubDone(brush):
+            a, b, start, duration, priority = expected.pop(0)
+            a = getattr(self.runner.video1, a)
+            b = getattr(self.runner.video1, b)
+
+            tr = self.runner.video1.transitions[(a, b)]
+
+            self.failUnlessEqual(b.start, start)
+            self.failUnlessEqual(a.start + a.duration - start,
+                duration)
+            self.failUnlessEqual(tr.start, start)
+            self.failUnlessEqual(tr.duration, duration)
+            self.failUnlessEqual(tr.priority, 0)
+            self.failUnlessEqual(a.priority, 0)
+            self.failUnlessEqual(b.priority, 0)
+            nextMove()
+
+        self.runner.loadConfiguration(initial)
+        self.runner.connect("timeline-configured", timelineConfigured)
+
+        brush = Brush(self.runner)
+        brush.connect("scrub-done", scrubDone)
+
+        self.runner.run()
+
+    def testNoTransitionWhenMovingMultipleClips(self):
+        initial = Configuration()
+        initial.addSource(
+            "object1", 
+            test1, 
+            {
+                "start" : 0,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object2", 
+            test1,
+            {
+                "start" : 5 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object3",
+            test1,
+            {
+                "start" : 10 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+
+        moves = [
+            ("object1", 9 * gst.SECOND, 0),
+            ("object3", 1* gst.SECOND, 0),
+        ]
+
+        def timelineConfigured(runner):
+            nextMove()
+
+        def nextMove():
+            if moves:
+                self._cur_move = moves.pop(0)
+                other, start, priority = self._cur_move
+                context = MoveContext(self.runner.timeline, 
+                    self.runner.video1.object2,
+                        set([getattr(self.runner.video1, other)]))
+                brush.scrub(context, start, priority, steps=10)
+            else:
+                self.runner.shutDown()
+
+        def scrubDone(brush):
+            self.failUnlessEqual(self.runner.video1.transitions,
+                {})
+            initial.matches(self.runner)
+            nextMove()
+
+        self.runner.loadConfiguration(initial)
+        self.runner.connect("timeline-configured", timelineConfigured)
+
+        brush = Brush(self.runner)
+        brush.connect("scrub-done", scrubDone)
+
+        self.runner.run()
+
+    def testOverlapOnlyWithValidTransitions(self):
+        initial = Configuration()
+        initial.addSource(
+            "object1", 
+            test1, 
+            {
+                "start" : 0,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object2", 
+            test1,
+            {
+                "start" : 5 * gst.SECOND,
+                "duration" : 3 * gst.SECOND,
+                "priority" : 0,
+            })
+        initial.addSource(
+            "object3",
+            test1,
+            {
+                "start" : 8 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+                "priority" : 0,
+            })
+
+        phase2 = initial.clone()
+        phase2.updateSource(
+            "object2", 
+            props={
+                "start" : 4 * gst.SECOND,
+            })
+
+        phase3 = phase2.clone()
+        phase3.updateSource(
+            "object3",
+            props={
+                "duration" : 1 * gst.SECOND
+            })
+
+        phase4 = initial.clone()
+        phase4.updateSource(
+            "object2",
+            props={
+                "start" : 3 * gst.SECOND,
+            })
+        phase4.updateSource(
+            "object3",
+            props={
+                "start" : 5 * gst.SECOND,
+                "duration" : 5 * gst.SECOND,
+            })
+
+        moves = [
+            # [1------]    [3--[2==]]
+            (MoveContext, "object2", 9 * gst.SECOND, 0, initial, []),
+
+            # [1--[2=]]    [3-------]
+            (MoveContext, "object2", 1 * gst.SECOND, 0, initial, []),
+
+            # [1------]    [3-------]
+            #        [2--]
+            (MoveContext, "object2", 4 * gst.SECOND, 0, phase2, 
+                [("object1", "object2")]),
+
+            # Activates overlap prevention
+            # [1------]
+            #      [3-------]
+            #        [2--]
+
+            (MoveContext, "object3", 3 * gst.SECOND, 0, phase2,
+                [("object1", "object2")]),
+
+            # [1------]  [3-]
+            #        [2--]
+            (TrimEndContext, "object3", 9 * gst.SECOND, 0, phase3,
+                [("object1", "object2")]),
+
+            # Activates overlap prevention
+            # [1------]    
+            #        [3-]
+            #        [2--]
+            (MoveContext, "object3", 4 * gst.SECOND, 0, phase3,
+                [("object1", "object2")]),
+
+            # Activates overlap prevention
+            # [1------]
+            #       [3]
+            #        [2--]
+            (MoveContext, "object3", long(3.5 * gst.SECOND), 0, phase3,
+                [("object1", "object2")]),
+
+            # Activates overlap prevention
+            # [1      ]
+            #         [3]
+            #        [2  ]
+            (MoveContext, "object3", long(4.5 * gst.SECOND), 0,
+                phase3, [("object1", "object2")]),
+
+            # Next few commands build this arrangement
+            # [1      ]
+            #     [2    ]
+            #          [3   ]
+
+            (MoveContext, "object2", 3 * gst.SECOND, 0,
+                None, None),
+            (MoveContext, "object3", 5 * gst.SECOND, 0,
+                None, None),
+            (TrimEndContext, "object3", 10 * gst.SECOND, 0,
+                phase4, [("object1", "object2"), ("object2",
+                    "object3")]),
+
+            # Activates Overlap Prevention
+            # [1      ]
+            #     [2    ]
+            #       [3   ]
+
+            (MoveContext, "object3", 4 * gst.SECOND, 0,
+                phase4, [("object1", "object2"),
+                    ("object2", "object3")]),
+
+        ]
+        
+        nmoves = len(moves)
+
+        def timelineConfigured(runner):
+            nextMove()
+
+        def nextMove():
+            if moves:
+                print "cur_move: %d/%d" % (nmoves - len(moves) + 1, nmoves)
+                self._cur_move = moves.pop(0)
+                context, focus, start, priority, config, trans = self._cur_move
+                obj = getattr(self.runner.video1, focus)
+                context = context(self.runner.timeline, 
+                     obj, set())
+                brush.scrub(context, start, priority, steps=10)
+            else:
+                self.runner.shutDown()
+
+        def scrubDone(brush):
+            connect, focus, stream, priority, config, trans = self._cur_move
+
+            if config:
+                config.matches(self.runner)
+
+            if trans:
+                expected = set([(getattr(self.runner.video1, a),
+                    getattr(self.runner.video1, b)) for a, b in
+                        trans])
+
+                self.failUnlessEqual(set(self.runner.video1.transitions.keys()),
+                   expected)
+            nextMove()
+
+        self.runner.loadConfiguration(initial)
+        self.runner.connect("timeline-configured", timelineConfigured)
+
+        brush = Brush(self.runner)
+        brush.connect("scrub-done", scrubDone)
+
+        self.runner.run()
+
+    def testSaveAndLoadWithTransitions(self):
+        pass
+
+# Readable code ends here <-
+
+if not os.getenv("TEST_INTEGRATION"):
+    import inspect
+
+    for symbol_name in list(globals().keys()):
+        symbol = globals()[symbol_name]
+        if inspect.isclass(symbol) and symbol.__name__.startswith('Test') \
+                and symbol.__module__ == __name__:
+            del globals()[symbol_name]
+
 if __name__ == "__main__":
     unittest.main()
